@@ -1,379 +1,335 @@
 -- mods/default/crafting.lua
 
 minetest.register_craft({
-	output = 'default:wood 4',
+	output = "default:wood 4",
 	recipe = {
-		{'default:tree'},
+		{"default:tree"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:junglewood 4',
+	output = "default:junglewood 4",
 	recipe = {
-		{'default:jungletree'},
+		{"default:jungletree"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pine_wood 4',
+	output = "default:pine_wood 4",
 	recipe = {
-		{'default:pine_tree'},
+		{"default:pine_tree"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:acacia_wood 4',
+	output = "default:acacia_wood 4",
 	recipe = {
-		{'default:acacia_tree'},
+		{"default:acacia_tree"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:aspen_wood 4',
+	output = "default:aspen_wood 4",
 	recipe = {
-		{'default:aspen_tree'},
+		{"default:aspen_tree"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:wood',
+	output = "default:wood",
 	recipe = {
-		{'default:bush_stem'},
+		{"default:bush_stem"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:acacia_wood',
+	output = "default:acacia_wood",
 	recipe = {
-		{'default:acacia_bush_stem'},
+		{"default:acacia_bush_stem"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:stick 4',
+	output = "default:pine_wood",
 	recipe = {
-		{'group:wood'},
+		{"default:pine_bush_stem"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:sign_wall_steel 3',
+	output = "default:stick 4",
 	recipe = {
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
-		{'', 'group:stick', ''},
+		{"group:wood"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:sign_wall_wood 3',
+	output = "default:sign_wall_steel 3",
 	recipe = {
-		{'group:wood', 'group:wood', 'group:wood'},
-		{'group:wood', 'group:wood', 'group:wood'},
-		{'', 'group:stick', ''},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:torch 4',
+	output = "default:sign_wall_wood 3",
 	recipe = {
-		{'default:coal_lump'},
-		{'group:stick'},
+		{"group:wood", "group:wood", "group:wood"},
+		{"group:wood", "group:wood", "group:wood"},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pick_wood',
+	output = "default:torch 4",
 	recipe = {
-		{'group:wood', 'group:wood', 'group:wood'},
-		{'', 'group:stick', ''},
-		{'', 'group:stick', ''},
+		{"default:coal_lump"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pick_stone',
+	output = "default:pick_wood",
 	recipe = {
-		{'group:stone', 'group:stone', 'group:stone'},
-		{'', 'group:stick', ''},
-		{'', 'group:stick', ''},
+		{"group:wood", "group:wood", "group:wood"},
+		{"", "group:stick", ""},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pick_steel',
+	output = "default:pick_stone",
 	recipe = {
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
-		{'', 'group:stick', ''},
-		{'', 'group:stick', ''},
+		{"group:stone", "group:stone", "group:stone"},
+		{"", "group:stick", ""},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pick_bronze',
+	output = "default:pick_steel",
 	recipe = {
-		{'default:bronze_ingot', 'default:bronze_ingot', 'default:bronze_ingot'},
-		{'', 'group:stick', ''},
-		{'', 'group:stick', ''},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
+		{"", "group:stick", ""},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pick_mese',
+	output = "default:pick_bronze",
 	recipe = {
-		{'default:mese_crystal', 'default:mese_crystal', 'default:mese_crystal'},
-		{'', 'group:stick', ''},
-		{'', 'group:stick', ''},
+		{"default:bronze_ingot", "default:bronze_ingot", "default:bronze_ingot"},
+		{"", "group:stick", ""},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:pick_diamond',
+	output = "default:pick_mese",
 	recipe = {
-		{'default:diamond', 'default:diamond', 'default:diamond'},
-		{'', 'group:stick', ''},
-		{'', 'group:stick', ''},
+		{"default:mese_crystal", "default:mese_crystal", "default:mese_crystal"},
+		{"", "group:stick", ""},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:shovel_wood',
+	output = "default:pick_diamond",
 	recipe = {
-		{'group:wood'},
-		{'group:stick'},
-		{'group:stick'},
+		{"default:diamond", "default:diamond", "default:diamond"},
+		{"", "group:stick", ""},
+		{"", "group:stick", ""},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:shovel_stone',
+	output = "default:shovel_wood",
 	recipe = {
-		{'group:stone'},
-		{'group:stick'},
-		{'group:stick'},
+		{"group:wood"},
+		{"group:stick"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:shovel_steel',
+	output = "default:shovel_stone",
 	recipe = {
-		{'default:steel_ingot'},
-		{'group:stick'},
-		{'group:stick'},
+		{"group:stone"},
+		{"group:stick"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:shovel_bronze',
+	output = "default:shovel_steel",
 	recipe = {
-		{'default:bronze_ingot'},
-		{'group:stick'},
-		{'group:stick'},
+		{"default:steel_ingot"},
+		{"group:stick"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:shovel_mese',
+	output = "default:shovel_bronze",
 	recipe = {
-		{'default:mese_crystal'},
-		{'group:stick'},
-		{'group:stick'},
+		{"default:bronze_ingot"},
+		{"group:stick"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:shovel_diamond',
+	output = "default:shovel_mese",
 	recipe = {
-		{'default:diamond'},
-		{'group:stick'},
-		{'group:stick'},
+		{"default:mese_crystal"},
+		{"group:stick"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_wood',
+	output = "default:shovel_diamond",
 	recipe = {
-		{'group:wood', 'group:wood'},
-		{'group:wood', 'group:stick'},
-		{'', 'group:stick'},
+		{"default:diamond"},
+		{"group:stick"},
+		{"group:stick"},
+	}
+})
+
+-- Axes
+-- Recipes face left to match appearence in textures and inventory
+
+minetest.register_craft({
+	output = "default:axe_wood",
+	recipe = {
+		{"group:wood", "group:wood"},
+		{"group:wood", "group:stick"},
+		{"", "group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_stone',
+	output = "default:axe_stone",
 	recipe = {
-		{'group:stone', 'group:stone'},
-		{'group:stone', 'group:stick'},
-		{'', 'group:stick'},
+		{"group:stone", "group:stone"},
+		{"group:stone", "group:stick"},
+		{"", "group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_steel',
+	output = "default:axe_steel",
 	recipe = {
-		{'default:steel_ingot', 'default:steel_ingot'},
-		{'default:steel_ingot', 'group:stick'},
-		{'', 'group:stick'},
+		{"default:steel_ingot", "default:steel_ingot"},
+		{"default:steel_ingot", "group:stick"},
+		{"", "group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_bronze',
+	output = "default:axe_bronze",
 	recipe = {
-		{'default:bronze_ingot', 'default:bronze_ingot'},
-		{'default:bronze_ingot', 'group:stick'},
-		{'', 'group:stick'},
+		{"default:bronze_ingot", "default:bronze_ingot"},
+		{"default:bronze_ingot", "group:stick"},
+		{"", "group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_mese',
+	output = "default:axe_mese",
 	recipe = {
-		{'default:mese_crystal', 'default:mese_crystal'},
-		{'default:mese_crystal', 'group:stick'},
-		{'', 'group:stick'},
+		{"default:mese_crystal", "default:mese_crystal"},
+		{"default:mese_crystal", "group:stick"},
+		{"", "group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_diamond',
+	output = "default:axe_diamond",
 	recipe = {
-		{'default:diamond', 'default:diamond'},
-		{'default:diamond', 'group:stick'},
-		{'', 'group:stick'},
+		{"default:diamond", "default:diamond"},
+		{"default:diamond", "group:stick"},
+		{"", "group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_wood',
+	output = "default:sword_wood",
 	recipe = {
-		{'group:wood', 'group:wood'},
-		{'group:stick', 'group:wood'},
-		{'group:stick',''},
+		{"group:wood"},
+		{"group:wood"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_stone',
+	output = "default:sword_stone",
 	recipe = {
-		{'group:stone', 'group:stone'},
-		{'group:stick', 'group:stone'},
-		{'group:stick', ''},
+		{"group:stone"},
+		{"group:stone"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_steel',
+	output = "default:sword_steel",
 	recipe = {
-		{'default:steel_ingot', 'default:steel_ingot'},
-		{'group:stick', 'default:steel_ingot'},
-		{'group:stick', ''},
+		{"default:steel_ingot"},
+		{"default:steel_ingot"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_bronze',
+	output = "default:sword_bronze",
 	recipe = {
-		{'default:bronze_ingot', 'default:bronze_ingot'},
-		{'group:stick', 'default:bronze_ingot'},
-		{'group:stick', ''},
+		{"default:bronze_ingot"},
+		{"default:bronze_ingot"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_mese',
+	output = "default:sword_mese",
 	recipe = {
-		{'default:mese_crystal', 'default:mese_crystal'},
-		{'group:stick', 'default:mese_crystal'},
-		{'group:stick', ''},
+		{"default:mese_crystal"},
+		{"default:mese_crystal"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:axe_diamond',
+	output = "default:sword_diamond",
 	recipe = {
-		{'default:diamond', 'default:diamond'},
-		{'group:stick', 'default:diamond'},
-		{'group:stick', ''},
+		{"default:diamond"},
+		{"default:diamond"},
+		{"group:stick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:sword_wood',
+	output = "default:skeleton_key",
 	recipe = {
-		{'group:wood'},
-		{'group:wood'},
-		{'group:stick'},
+		{"default:gold_ingot"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:sword_stone',
+	output = "default:chest",
 	recipe = {
-		{'group:stone'},
-		{'group:stone'},
-		{'group:stick'},
+		{"group:wood", "group:wood", "group:wood"},
+		{"group:wood", "", "group:wood"},
+		{"group:wood", "group:wood", "group:wood"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:sword_steel',
+	output = "default:chest_locked",
 	recipe = {
-		{'default:steel_ingot'},
-		{'default:steel_ingot'},
-		{'group:stick'},
-	}
-})
-
-minetest.register_craft({
-	output = 'default:sword_bronze',
-	recipe = {
-		{'default:bronze_ingot'},
-		{'default:bronze_ingot'},
-		{'group:stick'},
-	}
-})
-
-minetest.register_craft({
-	output = 'default:sword_mese',
-	recipe = {
-		{'default:mese_crystal'},
-		{'default:mese_crystal'},
-		{'group:stick'},
-	}
-})
-
-minetest.register_craft({
-	output = 'default:sword_diamond',
-	recipe = {
-		{'default:diamond'},
-		{'default:diamond'},
-		{'group:stick'},
-	}
-})
-
-minetest.register_craft({
-	output = 'default:skeleton_key',
-	recipe = {
-		{'default:gold_ingot'},
-	}
-})
-
-minetest.register_craft({
-	output = 'default:chest',
-	recipe = {
-		{'group:wood', 'group:wood', 'group:wood'},
-		{'group:wood', '', 'group:wood'},
-		{'group:wood', 'group:wood', 'group:wood'},
-	}
-})
-
-minetest.register_craft({
-	output = 'default:chest_locked',
-	recipe = {
-		{'group:wood', 'group:wood', 'group:wood'},
-		{'group:wood', 'default:steel_ingot', 'group:wood'},
-		{'group:wood', 'group:wood', 'group:wood'},
+		{"group:wood", "group:wood", "group:wood"},
+		{"group:wood", "default:steel_ingot", "group:wood"},
+		{"group:wood", "group:wood", "group:wood"},
 	}
 })
 
@@ -384,59 +340,59 @@ minetest.register_craft( {
 })
 
 minetest.register_craft({
-	output = 'default:furnace',
+	output = "default:furnace",
 	recipe = {
-		{'group:stone', 'group:stone', 'group:stone'},
-		{'group:stone', '', 'group:stone'},
-		{'group:stone', 'group:stone', 'group:stone'},
+		{"group:stone", "group:stone", "group:stone"},
+		{"group:stone", "", "group:stone"},
+		{"group:stone", "group:stone", "group:stone"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:coalblock',
+	output = "default:coalblock",
 	recipe = {
-		{'default:coal_lump', 'default:coal_lump', 'default:coal_lump'},
-		{'default:coal_lump', 'default:coal_lump', 'default:coal_lump'},
-		{'default:coal_lump', 'default:coal_lump', 'default:coal_lump'},
+		{"default:coal_lump", "default:coal_lump", "default:coal_lump"},
+		{"default:coal_lump", "default:coal_lump", "default:coal_lump"},
+		{"default:coal_lump", "default:coal_lump", "default:coal_lump"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:coal_lump 9',
+	output = "default:coal_lump 9",
 	recipe = {
-		{'default:coalblock'},
+		{"default:coalblock"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:steelblock',
+	output = "default:steelblock",
 	recipe = {
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:steel_ingot 9',
+	output = "default:steel_ingot 9",
 	recipe = {
-		{'default:steelblock'},
+		{"default:steelblock"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:copperblock',
+	output = "default:copperblock",
 	recipe = {
-		{'default:copper_ingot', 'default:copper_ingot', 'default:copper_ingot'},
-		{'default:copper_ingot', 'default:copper_ingot', 'default:copper_ingot'},
-		{'default:copper_ingot', 'default:copper_ingot', 'default:copper_ingot'},
+		{"default:copper_ingot", "default:copper_ingot", "default:copper_ingot"},
+		{"default:copper_ingot", "default:copper_ingot", "default:copper_ingot"},
+		{"default:copper_ingot", "default:copper_ingot", "default:copper_ingot"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:copper_ingot 9',
+	output = "default:copper_ingot 9",
 	recipe = {
-		{'default:copperblock'},
+		{"default:copperblock"},
 	}
 })
 
@@ -466,50 +422,50 @@ minetest.register_craft({
 })
 
 minetest.register_craft({
-	output = 'default:bronzeblock',
+	output = "default:bronzeblock",
 	recipe = {
-		{'default:bronze_ingot', 'default:bronze_ingot', 'default:bronze_ingot'},
-		{'default:bronze_ingot', 'default:bronze_ingot', 'default:bronze_ingot'},
-		{'default:bronze_ingot', 'default:bronze_ingot', 'default:bronze_ingot'},
+		{"default:bronze_ingot", "default:bronze_ingot", "default:bronze_ingot"},
+		{"default:bronze_ingot", "default:bronze_ingot", "default:bronze_ingot"},
+		{"default:bronze_ingot", "default:bronze_ingot", "default:bronze_ingot"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:bronze_ingot 9',
+	output = "default:bronze_ingot 9",
 	recipe = {
-		{'default:bronzeblock'},
+		{"default:bronzeblock"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:goldblock',
+	output = "default:goldblock",
 	recipe = {
-		{'default:gold_ingot', 'default:gold_ingot', 'default:gold_ingot'},
-		{'default:gold_ingot', 'default:gold_ingot', 'default:gold_ingot'},
-		{'default:gold_ingot', 'default:gold_ingot', 'default:gold_ingot'},
+		{"default:gold_ingot", "default:gold_ingot", "default:gold_ingot"},
+		{"default:gold_ingot", "default:gold_ingot", "default:gold_ingot"},
+		{"default:gold_ingot", "default:gold_ingot", "default:gold_ingot"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:gold_ingot 9',
+	output = "default:gold_ingot 9",
 	recipe = {
-		{'default:goldblock'},
+		{"default:goldblock"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:diamondblock',
+	output = "default:diamondblock",
 	recipe = {
-		{'default:diamond', 'default:diamond', 'default:diamond'},
-		{'default:diamond', 'default:diamond', 'default:diamond'},
-		{'default:diamond', 'default:diamond', 'default:diamond'},
+		{"default:diamond", "default:diamond", "default:diamond"},
+		{"default:diamond", "default:diamond", "default:diamond"},
+		{"default:diamond", "default:diamond", "default:diamond"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:diamond 9',
+	output = "default:diamond 9",
 	recipe = {
-		{'default:diamondblock'},
+		{"default:diamondblock"},
 	}
 })
 
@@ -610,57 +566,57 @@ minetest.register_craft({
 })
 
 minetest.register_craft({
-	output = 'default:clay',
+	output = "default:clay",
 	recipe = {
-		{'default:clay_lump', 'default:clay_lump'},
-		{'default:clay_lump', 'default:clay_lump'},
+		{"default:clay_lump", "default:clay_lump"},
+		{"default:clay_lump", "default:clay_lump"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:clay_lump 4',
+	output = "default:clay_lump 4",
 	recipe = {
-		{'default:clay'},
+		{"default:clay"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:brick',
+	output = "default:brick",
 	recipe = {
-		{'default:clay_brick', 'default:clay_brick'},
-		{'default:clay_brick', 'default:clay_brick'},
+		{"default:clay_brick", "default:clay_brick"},
+		{"default:clay_brick", "default:clay_brick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:clay_brick 4',
+	output = "default:clay_brick 4",
 	recipe = {
-		{'default:brick'},
+		{"default:brick"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:paper',
+	output = "default:paper",
 	recipe = {
-		{'default:papyrus', 'default:papyrus', 'default:papyrus'},
+		{"default:papyrus", "default:papyrus", "default:papyrus"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:book',
+	output = "default:book",
 	recipe = {
-		{'default:paper'},
-		{'default:paper'},
-		{'default:paper'},
+		{"default:paper"},
+		{"default:paper"},
+		{"default:paper"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:bookshelf',
+	output = "default:bookshelf",
 	recipe = {
-		{'group:wood', 'group:wood', 'group:wood'},
-		{'default:book', 'default:book', 'default:book'},
-		{'group:wood', 'group:wood', 'group:wood'},
+		{"group:wood", "group:wood", "group:wood"},
+		{"default:book", "default:book", "default:book"},
+		{"group:wood", "group:wood", "group:wood"},
 	}
 })
 
@@ -674,34 +630,34 @@ minetest.register_craft({
 })
 
 minetest.register_craft({
-	output = 'default:ladder_steel 15',
+	output = "default:ladder_steel 15",
 	recipe = {
-		{'default:steel_ingot', '', 'default:steel_ingot'},
-		{'default:steel_ingot', 'default:steel_ingot', 'default:steel_ingot'},
-		{'default:steel_ingot', '', 'default:steel_ingot'},
+		{"default:steel_ingot", "", "default:steel_ingot"},
+		{"default:steel_ingot", "default:steel_ingot", "default:steel_ingot"},
+		{"default:steel_ingot", "", "default:steel_ingot"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:mese',
+	output = "default:mese",
 	recipe = {
-		{'default:mese_crystal', 'default:mese_crystal', 'default:mese_crystal'},
-		{'default:mese_crystal', 'default:mese_crystal', 'default:mese_crystal'},
-		{'default:mese_crystal', 'default:mese_crystal', 'default:mese_crystal'},
+		{"default:mese_crystal", "default:mese_crystal", "default:mese_crystal"},
+		{"default:mese_crystal", "default:mese_crystal", "default:mese_crystal"},
+		{"default:mese_crystal", "default:mese_crystal", "default:mese_crystal"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:mese_crystal 9',
+	output = "default:mese_crystal 9",
 	recipe = {
-		{'default:mese'},
+		{"default:mese"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:mese_crystal_fragment 9',
+	output = "default:mese_crystal_fragment 9",
 	recipe = {
-		{'default:mese_crystal'},
+		{"default:mese_crystal"},
 	}
 })
 
@@ -715,10 +671,10 @@ minetest.register_craft({
 })
 
 minetest.register_craft({
-	output = 'default:meselamp',
+	output = "default:meselamp",
 	recipe = {
-		{'default:glass'},
-		{'default:mese_crystal'},
+		{"default:glass"},
+		{"default:mese_crystal"},
 	}
 })
 
@@ -732,96 +688,117 @@ minetest.register_craft({
 })
 
 minetest.register_craft({
-	output = 'default:obsidian_shard 9',
+	output = "default:obsidian_shard 9",
 	recipe = {
-		{'default:obsidian'}
+		{"default:obsidian"}
 	}
 })
 
 minetest.register_craft({
-	output = 'default:obsidian',
+	output = "default:obsidian",
 	recipe = {
-		{'default:obsidian_shard', 'default:obsidian_shard', 'default:obsidian_shard'},
-		{'default:obsidian_shard', 'default:obsidian_shard', 'default:obsidian_shard'},
-		{'default:obsidian_shard', 'default:obsidian_shard', 'default:obsidian_shard'},
+		{"default:obsidian_shard", "default:obsidian_shard", "default:obsidian_shard"},
+		{"default:obsidian_shard", "default:obsidian_shard", "default:obsidian_shard"},
+		{"default:obsidian_shard", "default:obsidian_shard", "default:obsidian_shard"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:obsidianbrick 4',
+	output = "default:obsidianbrick 4",
 	recipe = {
-		{'default:obsidian', 'default:obsidian'},
-		{'default:obsidian', 'default:obsidian'}
+		{"default:obsidian", "default:obsidian"},
+		{"default:obsidian", "default:obsidian"}
 	}
 })
 
 minetest.register_craft({
-	output = 'default:obsidian_block 9',
+	output = "default:obsidian_block 9",
 	recipe = {
-		{'default:obsidian', 'default:obsidian', 'default:obsidian'},
-		{'default:obsidian', 'default:obsidian', 'default:obsidian'},
-		{'default:obsidian', 'default:obsidian', 'default:obsidian'},
+		{"default:obsidian", "default:obsidian", "default:obsidian"},
+		{"default:obsidian", "default:obsidian", "default:obsidian"},
+		{"default:obsidian", "default:obsidian", "default:obsidian"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:stonebrick 4',
+	output = "default:stonebrick 4",
 	recipe = {
-		{'default:stone', 'default:stone'},
-		{'default:stone', 'default:stone'},
+		{"default:stone", "default:stone"},
+		{"default:stone", "default:stone"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:stone_block 9',
+	output = "default:stone_block 9",
 	recipe = {
-		{'default:stone', 'default:stone', 'default:stone'},
-		{'default:stone', 'default:stone', 'default:stone'},
-		{'default:stone', 'default:stone', 'default:stone'},
+		{"default:stone", "default:stone", "default:stone"},
+		{"default:stone", "default:stone", "default:stone"},
+		{"default:stone", "default:stone", "default:stone"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:desert_stonebrick 4',
+	output = "default:desert_stonebrick 4",
 	recipe = {
-		{'default:desert_stone', 'default:desert_stone'},
-		{'default:desert_stone', 'default:desert_stone'},
+		{"default:desert_stone", "default:desert_stone"},
+		{"default:desert_stone", "default:desert_stone"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:desert_stone_block 9',
+	output = "default:desert_stone_block 9",
 	recipe = {
-		{'default:desert_stone', 'default:desert_stone', 'default:desert_stone'},
-		{'default:desert_stone', 'default:desert_stone', 'default:desert_stone'},
-		{'default:desert_stone', 'default:desert_stone', 'default:desert_stone'},
+		{"default:desert_stone", "default:desert_stone", "default:desert_stone"},
+		{"default:desert_stone", "default:desert_stone", "default:desert_stone"},
+		{"default:desert_stone", "default:desert_stone", "default:desert_stone"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:snowblock',
+	output = "default:snowblock",
 	recipe = {
-		{'default:snow', 'default:snow', 'default:snow'},
-		{'default:snow', 'default:snow', 'default:snow'},
-		{'default:snow', 'default:snow', 'default:snow'},
+		{"default:snow", "default:snow", "default:snow"},
+		{"default:snow", "default:snow", "default:snow"},
+		{"default:snow", "default:snow", "default:snow"},
 	}
 })
 
 minetest.register_craft({
-	output = 'default:snow 9',
+	output = "default:snow 9",
 	recipe = {
-		{'default:snowblock'},
+		{"default:snowblock"},
 	}
 })
 
+minetest.register_craft({
+	output = "default:emergent_jungle_sapling",
+	recipe = {
+		{"default:junglesapling", "default:junglesapling", "default:junglesapling"},
+		{"default:junglesapling", "default:junglesapling", "default:junglesapling"},
+		{"default:junglesapling", "default:junglesapling", "default:junglesapling"},
+	}
+})
+
+minetest.register_craft({
+	output = "default:large_cactus_seedling",
+	recipe = {
+		{"", "default:cactus", ""},
+		{"default:cactus", "default:cactus", "default:cactus"},
+		{"", "default:cactus", ""},
+	}
+})
+
+
 --
 -- Crafting (tool repair)
 --
+
 minetest.register_craft({
 	type = "toolrepair",
 	additional_wear = -0.02,
 })
 
+
 --
 -- Cooking recipes
 --
@@ -887,24 +864,25 @@ minetest.register_craft({
 })
 
 minetest.register_craft({
-	type = 'cooking',
-	output = 'default:gold_ingot',
-	recipe = 'default:skeleton_key',
+	type = "cooking",
+	output = "default:gold_ingot",
+	recipe = "default:skeleton_key",
 	cooktime = 5,
 })
 
 minetest.register_craft({
-	type = 'cooking',
-	output = 'default:gold_ingot',
-	recipe = 'default:key',
+	type = "cooking",
+	output = "default:gold_ingot",
+	recipe = "default:key",
 	cooktime = 5,
 })
 
+
 --
 -- Fuels
 --
 
--- Support use of group:tree
+-- Support use of group:tree, includes default:tree which has the same burn time
 minetest.register_craft({
 	type = "fuel",
 	recipe = "group:tree",
@@ -927,12 +905,6 @@ minetest.register_craft({
 	burntime = 26,
 })
 
-minetest.register_craft({
-	type = "fuel",
-	recipe = "default:tree",
-	burntime = 30,
-})
-
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:acacia_tree",
@@ -946,7 +918,7 @@ minetest.register_craft({
 })
 
 
--- Support use of group:wood
+-- Support use of group:wood, includes default:wood which has the same burn time
 minetest.register_craft({
 	type = "fuel",
 	recipe = "group:wood",
@@ -965,12 +937,6 @@ minetest.register_craft({
 	burntime = 6,
 })
 
-minetest.register_craft({
-	type = "fuel",
-	recipe = "default:wood",
-	burntime = 7,
-})
-
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:acacia_wood",
@@ -984,53 +950,59 @@ minetest.register_craft({
 })
 
 
--- Support use of group:sapling
+-- Support use of group:sapling, includes default:sapling which has the same burn time
 minetest.register_craft({
 	type = "fuel",
 	recipe = "group:sapling",
-	burntime = 10,
+	burntime = 5,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:bush_sapling",
-	burntime = 6,
+	burntime = 3,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:acacia_bush_sapling",
-	burntime = 7,
+	burntime = 4,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:pine_bush_sapling",
+	burntime = 2,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:aspen_sapling",
-	burntime = 8,
+	burntime = 4,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:pine_sapling",
-	burntime = 9,
-})
-
-minetest.register_craft({
-	type = "fuel",
-	recipe = "default:sapling",
-	burntime = 10,
+	burntime = 5,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:acacia_sapling",
-	burntime = 11,
+	burntime = 6,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:junglesapling",
-	burntime = 12,
+	burntime = 6,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:emergent_jungle_sapling",
+	burntime = 7,
 })
 
 
@@ -1065,6 +1037,37 @@ minetest.register_craft({
 })
 
 
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:fence_rail_aspen_wood",
+	burntime = 3,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:fence_rail_pine_wood",
+	burntime = 4,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:fence_rail_wood",
+	burntime = 5,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:fence_rail_acacia_wood",
+	burntime = 6,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:fence_rail_junglewood",
+	burntime = 7,
+})
+
+
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:bush_stem",
@@ -1077,16 +1080,22 @@ minetest.register_craft({
 	burntime = 8,
 })
 
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:pine_bush_stem",
+	burntime = 6,
+})
+
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:junglegrass",
-	burntime = 2,
+	burntime = 3,
 })
 
 minetest.register_craft({
 	type = "fuel",
 	recipe = "group:leaves",
-	burntime = 1,
+	burntime = 4,
 })
 
 minetest.register_craft({
@@ -1095,10 +1104,16 @@ minetest.register_craft({
 	burntime = 15,
 })
 
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:large_cactus_seedling",
+	burntime = 5,
+})
+
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:papyrus",
-	burntime = 1,
+	burntime = 3,
 })
 
 minetest.register_craft({
@@ -1110,7 +1125,7 @@ minetest.register_craft({
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:ladder_wood",
-	burntime = 2,
+	burntime = 7,
 })
 
 minetest.register_craft({
@@ -1143,12 +1158,6 @@ minetest.register_craft({
 	burntime = 30,
 })
 
-minetest.register_craft({
-	type = "fuel",
-	recipe = "default:apple",
-	burntime = 3,
-})
-
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:coal_lump",
@@ -1173,6 +1182,18 @@ minetest.register_craft({
 	burntime = 2,
 })
 
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:fern_1",
+	burntime = 2,
+})
+
+minetest.register_craft({
+	type = "fuel",
+	recipe = "default:marram_grass_1",
+	burntime = 2,
+})
+
 minetest.register_craft({
 	type = "fuel",
 	recipe = "default:paper",
