@@ -1,42 +1,25 @@
+-- default/torch.lua
 
---[[
+-- support for MT game translation.
+local S = default.get_translator
 
-Torch mod - formerly mod "Torches"
-======================
-
-(c) Copyright BlockMen (2013-2015)
-(C) Copyright sofar <sofar@foo-projects.org> (2016)
-
-This mod changes the default torch drawtype from "torchlike" to "mesh",
-giving the torch a three dimensional appearance. The mesh contains the
-proper pixel mapping to make the animation appear as a particle above
-the torch, while in fact the animation is just the texture of the mesh.
-
-
-License:
-~~~~~~~~
-(c) Copyright BlockMen (2013-2015)
-
-Textures and Meshes/Models:
-CC-BY 3.0 BlockMen
-Note that the models were entirely done from scratch by sofar.
-
-Code:
-Licensed under the GNU LGPL version 2.1 or higher.
-You can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License
-as published by the Free Software Foundation;
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-See LICENSE.txt and http://www.gnu.org/licenses/lgpl-2.1.txt
-
---]]
+local function on_flood(pos, oldnode, newnode)
+	minetest.add_item(pos, ItemStack("default:torch 1"))
+	-- Play flame-extinguish sound if liquid is not an 'igniter'
+	local nodedef = minetest.registered_items[newnode.name]
+	if not (nodedef and nodedef.groups and
+			nodedef.groups.igniter and nodedef.groups.igniter > 0) then
+		minetest.sound_play(
+			"default_cool_lava",
+			{pos = pos, max_hear_distance = 16, gain = 0.1}
+		)
+	end
+	-- Remove the torch node
+	return false
+end
 
 minetest.register_node("default:torch", {
-	description = "Torch",
+	description = S("Torch"),
 	drawtype = "mesh",
 	mesh = "torch_floor.obj",
 	inventory_image = "default_torch_on_floor.png",
@@ -84,7 +67,9 @@ minetest.register_node("default:torch", {
 		itemstack:set_name("default:torch")
 
 		return itemstack
-	end
+	end,
+	floodable = true,
+	on_flood = on_flood,
 })
 
 minetest.register_node("default:torch_wall", {
@@ -106,6 +91,8 @@ minetest.register_node("default:torch_wall", {
 		wall_side = {-1/2, -1/2, -1/8, -1/8, 1/8, 1/8},
 	},
 	sounds = default.node_sound_wood_defaults(),
+	floodable = true,
+	on_flood = on_flood,
 })
 
 minetest.register_node("default:torch_ceiling", {
@@ -127,6 +114,8 @@ minetest.register_node("default:torch_ceiling", {
 		wall_top = {-1/8, -1/16, -5/16, 1/8, 1/2, 1/8},
 	},
 	sounds = default.node_sound_wood_defaults(),
+	floodable = true,
+	on_flood = on_flood,
 })
 
 minetest.register_lbm({
