@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef CHAT_HEADER
-#define CHAT_HEADER
+#pragma once
 
 #include <string>
 #include <vector>
@@ -26,27 +25,26 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "irrlichttypes.h"
 #include "util/enriched_string.h"
+#include "settings.h"
 
 // Chat console related classes
 
 struct ChatLine
 {
 	// age in seconds
-	f32 age;
+	f32 age = 0.0f;
 	// name of sending player, or empty if sent by server
 	EnrichedString name;
 	// message text
 	EnrichedString text;
 
 	ChatLine(const std::wstring &a_name, const std::wstring &a_text):
-		age(0.0),
 		name(a_name),
 		text(a_text)
 	{
 	}
 
 	ChatLine(const EnrichedString &a_name, const EnrichedString &a_text):
-		age(0.0),
 		name(a_name),
 		text(a_text)
 	{
@@ -75,11 +73,11 @@ class ChatBuffer
 {
 public:
 	ChatBuffer(u32 scrollback);
-	~ChatBuffer();
+	~ChatBuffer() = default;
 
 	// Append chat line
 	// Removes oldest chat line if scrollback size is reached
-	void addLine(std::wstring name, std::wstring text);
+	void addLine(const std::wstring &name, const std::wstring &text);
 
 	// Remove all chat lines
 	void clear();
@@ -121,6 +119,7 @@ public:
 	u32 formatChatLine(const ChatLine& line, u32 cols,
 			std::vector<ChatFormattedLine>& destination) const;
 
+	void resize(u32 scrollback);
 protected:
 	s32 getTopScrollPos() const;
 	s32 getBottomScrollPos() const;
@@ -132,11 +131,11 @@ private:
 	std::vector<ChatLine> m_unformatted;
 
 	// Number of character columns in console
-	u32 m_cols;
+	u32 m_cols = 0;
 	// Number of character rows in console
-	u32 m_rows;
+	u32 m_rows = 0;
 	// Scroll position (console's top line index into m_formatted)
-	s32 m_scroll;
+	s32 m_scroll = 0;
 	// Array of formatted lines
 	std::vector<ChatFormattedLine> m_formatted;
 	// Empty formatted line, for error returns
@@ -147,14 +146,14 @@ class ChatPrompt
 {
 public:
 	ChatPrompt(const std::wstring &prompt, u32 history_limit);
-	~ChatPrompt();
+	~ChatPrompt() = default;
 
 	// Input character or string
 	void input(wchar_t ch);
 	void input(const std::wstring &str);
 
 	// Add a string to the history
-	void addToHistory(std::wstring line);
+	void addToHistory(const std::wstring &line);
 
 	// Get current line
 	std::wstring getLine() const { return m_line; }
@@ -166,7 +165,7 @@ public:
 	void clear();
 
 	// Replace the current line with the given text
-	std::wstring replace(std::wstring line);
+	std::wstring replace(const std::wstring &line);
 
 	// Select previous command from history
 	void historyPrev();
@@ -225,39 +224,39 @@ protected:
 
 private:
 	// Prompt prefix
-	std::wstring m_prompt;
+	std::wstring m_prompt = L"";
 	// Currently edited line
-	std::wstring m_line;
+	std::wstring m_line = L"";
 	// History buffer
 	std::vector<std::wstring> m_history;
 	// History index (0 <= m_history_index <= m_history.size())
-	u32 m_history_index;
+	u32 m_history_index = 0;
 	// Maximum number of history entries
 	u32 m_history_limit;
 
 	// Number of columns excluding columns reserved for the prompt
-	s32 m_cols;
+	s32 m_cols = 0;
 	// Start of visible portion (index into m_line)
-	s32 m_view;
+	s32 m_view = 0;
 	// Cursor (index into m_line)
-	s32 m_cursor;
+	s32 m_cursor = 0;
 	// Cursor length (length of selected portion of line)
-	s32 m_cursor_len;
+	s32 m_cursor_len = 0;
 
 	// Last nick completion start (index into m_line)
-	s32 m_nick_completion_start;
+	s32 m_nick_completion_start = 0;
 	// Last nick completion start (index into m_line)
-	s32 m_nick_completion_end;
+	s32 m_nick_completion_end = 0;
 };
 
 class ChatBackend
 {
 public:
 	ChatBackend();
-	~ChatBackend();
+	~ChatBackend() = default;
 
 	// Add chat message
-	void addMessage(std::wstring name, std::wstring text);
+	void addMessage(const std::wstring &name, std::wstring text);
 	// Parse and add unparsed chat message
 	void addUnparsedMessage(std::wstring line);
 
@@ -266,7 +265,7 @@ public:
 	// Get the recent messages buffer
 	ChatBuffer& getRecentBuffer();
 	// Concatenate all recent messages
-	EnrichedString getRecentChat();
+	EnrichedString getRecentChat() const;
 	// Get the console prompt
 	ChatPrompt& getPrompt();
 
@@ -284,11 +283,11 @@ public:
 	void scrollPageDown();
 	void scrollPageUp();
 
+	// Resize recent buffer based on settings
+	void applySettings();
+
 private:
 	ChatBuffer m_console_buffer;
 	ChatBuffer m_recent_buffer;
 	ChatPrompt m_prompt;
 };
-
-#endif
-
