@@ -108,30 +108,15 @@ Client::Client(
 		m_minimap = new Minimap(this);
 	}
 	m_cache_save_interval = g_settings->getU16("server_map_save_interval");
-
-	m_modding_enabled = g_settings->getBool("enable_client_modding");
-	// Only create the client script environment if client scripting is enabled by the
-	// client.
-	if (m_modding_enabled) {
-		m_script = new ClientScripting(this);
-		m_env.setScript(m_script);
-		m_script->setEnv(&m_env);
-	}
 }
 
 void Client::loadMods()
 {
-	// Don't load mods twice
-	if (m_mods_loaded) {
-		return;
-	}
-
+	// Don't load mods twice.
 	// If client scripting is disabled by the client, don't load builtin or
 	// client-provided mods.
-	if (!m_modding_enabled) {
-		warningstream << "Client side scripting is disabled by client." << std::endl;
+	if (m_mods_loaded || !g_settings->getBool("enable_client_modding"))
 		return;
-	}
 
 	// If client scripting is disabled by the server, don't load builtin or
 	// client-provided mods.
@@ -140,11 +125,13 @@ void Client::loadMods()
 	if (checkCSMRestrictionFlag(CSMRestrictionFlags::CSM_RF_LOAD_CLIENT_MODS)) {
 		warningstream << "Client-provided mod loading is disabled by server." <<
 			std::endl;
-		// This line is needed because builtin is not loaded
-		m_modding_enabled = false;
 		return;
 	}
 
+	m_script = new ClientScripting(this);
+	m_env.setScript(m_script);
+	m_script->setEnv(&m_env);
+
 	// Load builtin
 	scanModIntoMemory(BUILTIN_MOD_NAME, getBuiltinLuaPath());
 	m_script->loadModFromMemory(BUILTIN_MOD_NAME);
@@ -190,9 +177,15 @@ void Client::loadMods()
 	for (const ModSpec &mod : m_mods)
 		m_script->loadModFromMemory(mod.name);
 
+	// Mods are done loading. Unlock callbacks
+	m_mods_loaded = true;
+
 	// Run a callback when mods are loaded
 	m_script->on_mods_loaded();
-	m_mods_loaded = true;
+	if (m_state == LC_Ready)
+		m_script->on_client_ready(m_env.getLocalPlayer());
+	if (m_camera)
+		m_script->on_camera_ready(m_camera);
 }
 
 bool Client::checkBuiltinIntegrity()
@@ -244,7 +237,7 @@ const ModSpec* Client::getModSpec(const std::string &modname) const
 void Client::Stop()
 {
 	m_shutdown = true;
-	if (m_modding_enabled)
+	if (m_mods_loaded)
 		m_script->on_shutdown();
 	//request all client managed threads to stop
 	m_mesh_update_thread.stop();
@@ -254,7 +247,7 @@ void Client::Stop()
 		m_localdb->endSave();
 	}
 
-	if (m_modding_enabled)
+	if (m_mods_loaded)
 		delete m_script;
 }
 
@@ -371,7 +364,6 @@ void Client::step(float dtime)
 	*/
 	const float map_timer_and_unload_dtime = 5.25;
 	if(m_map_timer_and_unload_interval.step(dtime, map_timer_and_unload_dtime)) {
-		ScopeProfiler sp(g_profiler, "Client: map timer and unload");
 		std::vector<v3s16> deleted_blocks;
 		m_env.getMap().timerUpdate(map_timer_and_unload_dtime,
 			g_settings->getFloat("client_unload_unused_data_timeout"),
@@ -480,6 +472,7 @@ void Client::step(float dtime)
 	*/
 	{
 		int num_processed_meshes = 0;
+		std::vector<v3s16> blocks_to_ack;
 		while (!m_mesh_update_thread.m_queue_out.empty())
 		{
 			num_processed_meshes++;
@@ -518,14 +511,18 @@ void Client::step(float dtime)
 				m_minimap->addBlock(r.p, minimap_mapblock);
 
 			if (r.ack_block_to_server) {
-				/*
-					Acknowledge block
-					[0] u8 count
-					[1] v3s16 pos_0
-				*/
-				sendGotBlocks(r.p);
+				if (blocks_to_ack.size() == 255) {
+					sendGotBlocks(blocks_to_ack);
+					blocks_to_ack.clear();
+				}
+
+				blocks_to_ack.emplace_back(r.p);
 			}
 		}
+		if (blocks_to_ack.size() > 0) {
+				// Acknowledge block(s)
+				sendGotBlocks(blocks_to_ack);
+		}
 
 		if (num_processed_meshes > 0)
 			g_profiler->graphAdd("num_processed_meshes", num_processed_meshes);
@@ -558,8 +555,8 @@ void Client::step(float dtime)
 		if (count_after != count_before) {
 			// Do this every <interval> seconds after TOCLIENT_INVENTORY
 			// Reset the locally changed inventory to the authoritative inventory
-			m_env.getLocalPlayer()->inventory = *m_inventory_from_server;
-			m_inventory_updated = true;
+			player->inventory = *m_inventory_from_server;
+			m_update_wielded_item = true;
 		}
 	}
 
@@ -908,7 +905,7 @@ void writePlayerPos(LocalPlayer *myplayer, ClientMap *clientMap, NetworkPacket *
 	*pkt << fov << wanted_range;
 }
 
-void Client::interact(u8 action, const PointedThing& pointed)
+void Client::interact(InteractAction action, const PointedThing& pointed)
 {
 	if(m_state != LC_Ready) {
 		errorstream << "Client::interact() "
@@ -928,19 +925,12 @@ void Client::interact(u8 action, const PointedThing& pointed)
 		[5] u32 length of the next item (plen)
 		[9] serialized PointedThing
 		[9 + plen] player position information
-		actions:
-		0: start digging (from undersurface) or use
-		1: stop digging (all parameters ignored)
-		2: digging completed
-		3: place block or item (to abovesurface)
-		4: use item
-		5: perform secondary action of item
 	*/
 
 	NetworkPacket pkt(TOSERVER_INTERACT, 1 + 2 + 0);
 
-	pkt << action;
-	pkt << (u16)getPlayerItem();
+	pkt << (u8)action;
+	pkt << myplayer->getWieldIndex();
 
 	std::ostringstream tmp_os(std::ios::binary);
 	pointed.serialize(tmp_os);
@@ -1074,10 +1064,13 @@ void Client::sendDeletedBlocks(std::vector<v3s16> &blocks)
 	Send(&pkt);
 }
 
-void Client::sendGotBlocks(v3s16 block)
+void Client::sendGotBlocks(const std::vector<v3s16> &blocks)
 {
-	NetworkPacket pkt(TOSERVER_GOTBLOCKS, 1 + 6);
-	pkt << (u8) 1 << block;
+	NetworkPacket pkt(TOSERVER_GOTBLOCKS, 1 + 6 * blocks.size());
+	pkt << (u8) blocks.size();
+	for (const v3s16 &block : blocks)
+		pkt << block;
+
 	Send(&pkt);
 }
 
@@ -1200,7 +1193,7 @@ void Client::clearOutChatQueue()
 }
 
 void Client::sendChangePassword(const std::string &oldpassword,
-        const std::string &newpassword)
+	const std::string &newpassword)
 {
 	LocalPlayer *player = m_env.getLocalPlayer();
 	if (player == NULL)
@@ -1229,60 +1222,54 @@ void Client::sendRespawn()
 void Client::sendReady()
 {
 	NetworkPacket pkt(TOSERVER_CLIENT_READY,
-			1 + 1 + 1 + 1 + 2 + sizeof(char) * strlen(g_version_hash));
+			1 + 1 + 1 + 1 + 2 + sizeof(char) * strlen(g_version_hash) + 2);
 
 	pkt << (u8) VERSION_MAJOR << (u8) VERSION_MINOR << (u8) VERSION_PATCH
 		<< (u8) 0 << (u16) strlen(g_version_hash);
 
 	pkt.putRawString(g_version_hash, (u16) strlen(g_version_hash));
+	pkt << (u16)FORMSPEC_API_VERSION;
 	Send(&pkt);
 }
 
 void Client::sendPlayerPos()
 {
-	LocalPlayer *myplayer = m_env.getLocalPlayer();
-	if (!myplayer)
+	LocalPlayer *player = m_env.getLocalPlayer();
+	if (!player)
 		return;
 
 	ClientMap &map = m_env.getClientMap();
+	u8 camera_fov   = map.getCameraFov();
+	u8 wanted_range = map.getControl().wanted_range;
 
-	u8 camera_fov    = map.getCameraFov();
-	u8 wanted_range  = map.getControl().wanted_range;
+	// Save bandwidth by only updating position when
+	// player is not dead and something changed
 
-	// Save bandwidth by only updating position when something changed
-	if(myplayer->last_position        == myplayer->getPosition() &&
-			myplayer->last_speed        == myplayer->getSpeed()    &&
-			myplayer->last_pitch        == myplayer->getPitch()    &&
-			myplayer->last_yaw          == myplayer->getYaw()      &&
-			myplayer->last_keyPressed   == myplayer->keyPressed    &&
-			myplayer->last_camera_fov   == camera_fov              &&
-			myplayer->last_wanted_range == wanted_range)
+	// FIXME: This part causes breakages in mods like 3d_armor, and has been commented for now
+	// if (m_activeobjects_received && player->isDead())
+	//	return;
+
+	if (
+			player->last_position     == player->getPosition() &&
+			player->last_speed        == player->getSpeed()    &&
+			player->last_pitch        == player->getPitch()    &&
+			player->last_yaw          == player->getYaw()      &&
+			player->last_keyPressed   == player->keyPressed    &&
+			player->last_camera_fov   == camera_fov              &&
+			player->last_wanted_range == wanted_range)
 		return;
 
-	myplayer->last_position     = myplayer->getPosition();
-	myplayer->last_speed        = myplayer->getSpeed();
-	myplayer->last_pitch        = myplayer->getPitch();
-	myplayer->last_yaw          = myplayer->getYaw();
-	myplayer->last_keyPressed   = myplayer->keyPressed;
-	myplayer->last_camera_fov   = camera_fov;
-	myplayer->last_wanted_range = wanted_range;
+	player->last_position     = player->getPosition();
+	player->last_speed        = player->getSpeed();
+	player->last_pitch        = player->getPitch();
+	player->last_yaw          = player->getYaw();
+	player->last_keyPressed   = player->keyPressed;
+	player->last_camera_fov   = camera_fov;
+	player->last_wanted_range = wanted_range;
 
 	NetworkPacket pkt(TOSERVER_PLAYERPOS, 12 + 12 + 4 + 4 + 4 + 1 + 1);
 
-	writePlayerPos(myplayer, &map, &pkt);
-
-	Send(&pkt);
-}
-
-void Client::sendPlayerItem(u16 item)
-{
-	LocalPlayer *myplayer = m_env.getLocalPlayer();
-	if (!myplayer)
-		return;
-
-	NetworkPacket pkt(TOSERVER_PLAYERITEM, 2);
-
-	pkt << item;
+	writePlayerPos(player, &map, &pkt);
 
 	Send(&pkt);
 }
@@ -1318,7 +1305,7 @@ MapNode Client::getNode(v3s16 p, bool *is_valid_position)
 			return {};
 		}
 	}
-	return m_env.getMap().getNodeNoEx(p, is_valid_position);
+	return m_env.getMap().getNode(p, is_valid_position);
 }
 
 void Client::addNode(v3s16 p, MapNode n, bool remove_metadata)
@@ -1346,28 +1333,33 @@ void Client::setPlayerControl(PlayerControl &control)
 	player->control = control;
 }
 
-void Client::selectPlayerItem(u16 item)
+void Client::setPlayerItem(u16 item)
 {
-	m_playeritem = item;
-	m_inventory_updated = true;
-	sendPlayerItem(item);
+	m_env.getLocalPlayer()->setWieldIndex(item);
+	m_update_wielded_item = true;
+
+	NetworkPacket pkt(TOSERVER_PLAYERITEM, 2);
+	pkt << item;
+	Send(&pkt);
 }
 
-// Returns true if the inventory of the local player has been
-// updated from the server. If it is true, it is set to false.
-bool Client::getLocalInventoryUpdated()
+// Returns true once after the inventory of the local player
+// has been updated from the server.
+bool Client::updateWieldedItem()
 {
-	bool updated = m_inventory_updated;
-	m_inventory_updated = false;
-	return updated;
-}
+	if (!m_update_wielded_item)
+		return false;
+
+	m_update_wielded_item = false;
 
-// Copies the inventory of the local player to parameter
-void Client::getLocalInventory(Inventory &dst)
-{
 	LocalPlayer *player = m_env.getLocalPlayer();
 	assert(player);
-	dst = player->inventory;
+	if (auto *list = player->inventory.getList("main"))
+		list->setModified(false);
+	if (auto *list = player->inventory.getList("hand"))
+		list->setModified(false);
+
+	return true;
 }
 
 Inventory* Client::getInventory(const InventoryLocation &loc)
@@ -1510,7 +1502,7 @@ void Client::typeChatMessage(const std::wstring &message)
 		return;
 
 	// If message was consumed by script API, don't send it to server
-	if (m_modding_enabled && m_script->on_sending_message(wide_to_utf8(message)))
+	if (m_mods_loaded && m_script->on_sending_message(wide_to_utf8(message)))
 		return;
 
 	// Send to others
@@ -1706,9 +1698,8 @@ void Client::afterContentReceived()
 	m_state = LC_Ready;
 	sendReady();
 
-	if (g_settings->getBool("enable_client_modding")) {
+	if (m_mods_loaded)
 		m_script->on_client_ready(m_env.getLocalPlayer());
-	}
 
 	text = wgettext("Done!");
 	RenderingEngine::draw_load_screen(text, guienv, m_tsrc, 0, 100);
