@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef JOYSTICK_HEADER
-#define JOYSTICK_HEADER
+#pragma once
 
 #include "irrlichttypes_extrabloated.h"
 #include "keys.h"
@@ -52,7 +51,8 @@ struct JoystickCombination {
 
 struct JoystickButtonCmb : public JoystickCombination {
 
-	JoystickButtonCmb() {}
+	JoystickButtonCmb() = default;
+
 	JoystickButtonCmb(GameKeyType key, u32 filter_mask, u32 compare_mask) :
 		filter_mask(filter_mask),
 		compare_mask(compare_mask)
@@ -60,7 +60,7 @@ struct JoystickButtonCmb : public JoystickCombination {
 		this->key = key;
 	}
 
-	virtual ~JoystickButtonCmb() {}
+	virtual ~JoystickButtonCmb() = default;
 
 	virtual bool isTriggered(const irr::SEvent::SJoystickEvent &ev) const;
 
@@ -70,7 +70,8 @@ struct JoystickButtonCmb : public JoystickCombination {
 
 struct JoystickAxisCmb : public JoystickCombination {
 
-	JoystickAxisCmb() {}
+	JoystickAxisCmb() = default;
+
 	JoystickAxisCmb(GameKeyType key, u16 axis_to_compare, int direction, s16 thresh) :
 		axis_to_compare(axis_to_compare),
 		direction(direction),
@@ -79,9 +80,9 @@ struct JoystickAxisCmb : public JoystickCombination {
 		this->key = key;
 	}
 
-	virtual ~JoystickAxisCmb() {}
+	virtual ~JoystickAxisCmb() = default;
 
-	virtual bool isTriggered(const irr::SEvent::SJoystickEvent &ev) const;
+	bool isTriggered(const irr::SEvent::SJoystickEvent &ev) const override;
 
 	u16 axis_to_compare;
 
@@ -159,7 +160,7 @@ private:
 
 	s16 m_axes_vals[JA_COUNT];
 
-	u8 m_joystick_id;
+	u8 m_joystick_id = 0;
 
 	std::bitset<KeyType::INTERNAL_ENUM_COUNT> m_pressed_keys;
 
@@ -170,5 +171,3 @@ private:
 	std::bitset<KeyType::INTERNAL_ENUM_COUNT> m_past_pressed_keys;
 	std::bitset<KeyType::INTERNAL_ENUM_COUNT> m_past_released_keys;
 };
-
-#endif
