@@ -18,18 +18,17 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 */
 
 #include <sstream>
-
 #include "clientiface.h"
+#include "network/connection.h"
+#include "network/serveropcodes.h"
 #include "remoteplayer.h"
 #include "settings.h"
 #include "mapblock.h"
-#include "network/connection.h"
 #include "serverenvironment.h"
 #include "map.h"
 #include "emerge.h"
 #include "content_sao.h"              // TODO this is used for cleanup of only
 #include "log.h"
-#include "network/serveropcodes.h"
 #include "util/srp.h"
 #include "face_position_cache.h"
 
@@ -53,6 +52,17 @@ std::string ClientInterface::state2Name(ClientState state)
 	return statenames[state];
 }
 
+RemoteClient::RemoteClient() :
+	m_max_simul_sends(g_settings->getU16("max_simultaneous_block_sends_per_client")),
+	m_min_time_from_building(
+		g_settings->getFloat("full_block_send_enable_min_time_from_building")),
+	m_max_send_distance(g_settings->getS16("max_block_send_distance")),
+	m_block_optimize_distance(g_settings->getS16("block_send_optimize_distance")),
+	m_max_gen_distance(g_settings->getS16("max_block_generate_distance")),
+	m_occ_cull(g_settings->getBool("server_side_occlusion_culling"))
+{
+}
+
 void RemoteClient::ResendBlockIfOnWire(v3s16 p)
 {
 	// if this block is on wire, mark it for sending again as soon as possible
@@ -61,46 +71,61 @@ void RemoteClient::ResendBlockIfOnWire(v3s16 p)
 	}
 }
 
+LuaEntitySAO *getAttachedObject(PlayerSAO *sao, ServerEnvironment *env)
+{
+	if (!sao->isAttached())
+		return nullptr;
+
+	int id;
+	std::string bone;
+	v3f dummy;
+	sao->getAttachment(&id, &bone, &dummy, &dummy);
+	ServerActiveObject *ao = env->getActiveObject(id);
+	while (id && ao) {
+		ao->getAttachment(&id, &bone, &dummy, &dummy);
+		if (id)
+			ao = env->getActiveObject(id);
+	}
+	return dynamic_cast<LuaEntitySAO *>(ao);
+}
+
 void RemoteClient::GetNextBlocks (
 		ServerEnvironment *env,
 		EmergeManager * emerge,
 		float dtime,
 		std::vector<PrioritySortedBlockTransfer> &dest)
 {
-	DSTACK(FUNCTION_NAME);
-
-
 	// Increment timers
 	m_nothing_to_send_pause_timer -= dtime;
 	m_nearest_unsent_reset_timer += dtime;
 
-	if(m_nothing_to_send_pause_timer >= 0)
+	if (m_nothing_to_send_pause_timer >= 0)
 		return;
 
 	RemotePlayer *player = env->getPlayer(peer_id);
 	// This can happen sometimes; clients and players are not in perfect sync.
-	if (player == NULL)
+	if (!player)
 		return;
 
 	PlayerSAO *sao = player->getPlayerSAO();
-	if (sao == NULL)
+	if (!sao)
 		return;
 
 	// Won't send anything if already sending
-	if(m_blocks_sending.size() >= g_settings->getU16
-			("max_simultaneous_block_sends_per_client"))
-	{
+	if (m_blocks_sending.size() >= m_max_simul_sends) {
 		//infostream<<"Not sending any blocks, Queue full."<<std::endl;
 		return;
 	}
 
 	v3f playerpos = sao->getBasePosition();
-	v3f playerspeed = player->getSpeed();
+	// if the player is attached, get the velocity from the attached object
+	LuaEntitySAO *lsao = getAttachedObject(sao, env);
+	const v3f &playerspeed = lsao? lsao->getVelocity() : player->getSpeed();
 	v3f playerspeeddir(0,0,0);
-	if(playerspeed.getLength() > 1.0*BS)
+	if (playerspeed.getLength() > 1.0f * BS)
 		playerspeeddir = playerspeed / playerspeed.getLength();
 	// Predict to next block
-	v3f playerpos_predicted = playerpos + playerspeeddir*MAP_BLOCKSIZE*BS;
+	v3f playerpos_predicted = playerpos + playerspeeddir * (MAP_BLOCKSIZE * BS);
 
 	v3s16 center_nodepos = floatToInt(playerpos_predicted, BS);
 
@@ -109,8 +134,8 @@ void RemoteClient::GetNextBlocks (
 	// Camera position and direction
 	v3f camera_pos = sao->getEyePosition();
 	v3f camera_dir = v3f(0,0,1);
-	camera_dir.rotateYZBy(sao->getPitch());
-	camera_dir.rotateXZBy(sao->getYaw());
+	camera_dir.rotateYZBy(sao->getLookPitch());
+	camera_dir.rotateXZBy(sao->getRotation().Y);
 
 	/*infostream<<"camera_dir=("<<camera_dir.X<<","<<camera_dir.Y<<","
 			<<camera_dir.Z<<")"<<std::endl;*/
@@ -119,8 +144,7 @@ void RemoteClient::GetNextBlocks (
 		Get the starting value of the block finder radius.
 	*/
 
-	if(m_last_center != center)
-	{
+	if (m_last_center != center) {
 		m_nearest_unsent_d = 0;
 		m_last_center = center;
 	}
@@ -129,9 +153,8 @@ void RemoteClient::GetNextBlocks (
 			<<m_nearest_unsent_reset_timer<<std::endl;*/
 
 	// Reset periodically to workaround for some bugs or stuff
-	if(m_nearest_unsent_reset_timer > 20.0)
-	{
-		m_nearest_unsent_reset_timer = 0;
+	if (m_nearest_unsent_reset_timer > 20.0f) {
+		m_nearest_unsent_reset_timer = 0.0f;
 		m_nearest_unsent_d = 0;
 		//infostream<<"Resetting m_nearest_unsent_d for "
 		//		<<server->getPlayerName(peer_id)<<std::endl;
@@ -142,9 +165,7 @@ void RemoteClient::GetNextBlocks (
 
 	//infostream<<"d_start="<<d_start<<std::endl;
 
-	u16 max_simul_sends_setting = g_settings->getU16
-			("max_simultaneous_block_sends_per_client");
-	u16 max_simul_sends_usually = max_simul_sends_setting;
+	u16 max_simul_sends_usually = m_max_simul_sends;
 
 	/*
 		Check the time from last addNode/removeNode.
@@ -152,9 +173,7 @@ void RemoteClient::GetNextBlocks (
 		Decrease send rate if player is building stuff.
 	*/
 	m_time_from_building += dtime;
-	if(m_time_from_building < g_settings->getFloat(
-				"full_block_send_enable_min_time_from_building"))
-	{
+	if (m_time_from_building < m_min_time_from_building) {
 		max_simul_sends_usually
 			= LIMITED_MAX_SIMULTANEOUS_BLOCK_SENDS;
 	}
@@ -173,36 +192,51 @@ void RemoteClient::GetNextBlocks (
 	*/
 	s32 new_nearest_unsent_d = -1;
 
-	// get view range and camera fov from the client
-	s16 wanted_range = sao->getWantedRange();
+	// Get view range and camera fov (radians) from the client
+	s16 wanted_range = sao->getWantedRange() + 1;
 	float camera_fov = sao->getFov();
-	// if FOV, wanted_range are not available (old client), fall back to old default
-	if (wanted_range <= 0) wanted_range = 1000;
-	if (camera_fov <= 0) camera_fov = (72.0*M_PI/180) * 4./3.;
 
-	const s16 full_d_max = MYMIN(g_settings->getS16("max_block_send_distance"), wanted_range);
-	const s16 d_opt = MYMIN(g_settings->getS16("block_send_optimize_distance"), wanted_range);
+	// Distrust client-sent FOV and get server-set player object property
+	// zoom FOV (degrees) as a check to avoid hacked clients using FOV to load
+	// distant world.
+	// (zoom is disabled by value 0)
+	float prop_zoom_fov = sao->getZoomFOV() < 0.001f ?
+		0.0f :
+		std::max(camera_fov, sao->getZoomFOV() * core::DEGTORAD);
+
+	const s16 full_d_max = std::min(adjustDist(m_max_send_distance, prop_zoom_fov),
+		wanted_range);
+	const s16 d_opt = std::min(adjustDist(m_block_optimize_distance, prop_zoom_fov),
+		wanted_range);
 	const s16 d_blocks_in_sight = full_d_max * BS * MAP_BLOCKSIZE;
-	//infostream << "Fov from client " << camera_fov << " full_d_max " << full_d_max << std::endl;
 
 	s16 d_max = full_d_max;
-	s16 d_max_gen = MYMIN(g_settings->getS16("max_block_generate_distance"), wanted_range);
+	s16 d_max_gen = std::min(adjustDist(m_max_gen_distance, prop_zoom_fov),
+		wanted_range);
 
-	// Don't loop very much at a time
-	s16 max_d_increment_at_time = 2;
-	if(d_max > d_start + max_d_increment_at_time)
+	// Don't loop very much at a time, adjust with distance,
+	// do more work per RTT with greater distances.
+	s16 max_d_increment_at_time = full_d_max / 9 + 1;
+	if (d_max > d_start + max_d_increment_at_time)
 		d_max = d_start + max_d_increment_at_time;
 
+	// cos(angle between velocity and camera) * |velocity|
+	// Limit to 0.0f in case player moves backwards.
+	f32 dot = rangelim(camera_dir.dotProduct(playerspeed), 0.0f, 300.0f);
+
+	// Reduce the field of view when a player moves and looks forward.
+	// limit max fov effect to 50%, 60% at 20n/s fly speed
+	camera_fov = camera_fov / (1 + dot / 300.0f);
+
 	s32 nearest_emerged_d = -1;
 	s32 nearest_emergefull_d = -1;
 	s32 nearest_sent_d = -1;
 	//bool queue_is_full = false;
 
 	const v3s16 cam_pos_nodes = floatToInt(camera_pos, BS);
-	const bool occ_cull = g_settings->getBool("server_side_occlusion_culling");
 
 	s16 d;
-	for(d = d_start; d <= d_max; d++) {
+	for (d = d_start; d <= d_max; d++) {
 		/*
 			Get the border/face dot coordinates of a "d-radiused"
 			box
@@ -210,7 +244,7 @@ void RemoteClient::GetNextBlocks (
 		std::vector<v3s16> list = FacePositionCache::getFacePositions(d);
 
 		std::vector<v3s16>::iterator li;
-		for(li = list.begin(); li != list.end(); ++li) {
+		for (li = list.begin(); li != list.end(); ++li) {
 			v3s16 p = *li + center;
 
 			/*
@@ -225,8 +259,8 @@ void RemoteClient::GetNextBlocks (
 			u16 max_simul_dynamic = max_simul_sends_usually;
 
 			// If block is very close, allow full maximum
-			if(d <= BLOCK_SEND_DISABLE_LIMITS_MAX_D)
-				max_simul_dynamic = max_simul_sends_setting;
+			if (d <= BLOCK_SEND_DISABLE_LIMITS_MAX_D)
+				max_simul_dynamic = m_max_simul_sends;
 
 			// Don't select too many blocks for sending
 			if (num_blocks_selected >= max_simul_dynamic) {
@@ -251,22 +285,24 @@ void RemoteClient::GetNextBlocks (
 				Don't generate or send if not in sight
 				FIXME This only works if the client uses a small enough
 				FOV setting. The default of 72 degrees is fine.
+				Also retrieve a smaller view cone in the direction of the player's
+				movement.
+				(0.1 is about 4 degrees)
 			*/
-
 			f32 dist;
-			if (!isBlockInSight(p, camera_pos, camera_dir, camera_fov, d_blocks_in_sight, &dist)) {
+			if (!(isBlockInSight(p, camera_pos, camera_dir, camera_fov,
+						d_blocks_in_sight, &dist) ||
+					(playerspeed.getLength() > 1.0f * BS &&
+					isBlockInSight(p, camera_pos, playerspeeddir, 0.1f,
+						d_blocks_in_sight)))) {
 				continue;
 			}
 
 			/*
 				Don't send already sent blocks
 			*/
-			{
-				if(m_blocks_sent.find(p) != m_blocks_sent.end())
-				{
-					continue;
-				}
-			}
+			if (m_blocks_sent.find(p) != m_blocks_sent.end())
+				continue;
 
 			/*
 				Check if map has this block
@@ -275,19 +311,17 @@ void RemoteClient::GetNextBlocks (
 
 			bool surely_not_found_on_disk = false;
 			bool block_is_invalid = false;
-			if(block != NULL)
-			{
+			if (block) {
 				// Reset usage timer, this block will be of use in the future.
 				block->resetUsageTimer();
 
 				// Block is dummy if data doesn't exist.
 				// It means it has been not found from disk and not generated
-				if(block->isDummy())
-				{
+				if (block->isDummy()) {
 					surely_not_found_on_disk = true;
 				}
 
-				if(block->isGenerated() == false)
+				if (!block->isGenerated())
 					block_is_invalid = true;
 
 				/*
@@ -297,13 +331,12 @@ void RemoteClient::GetNextBlocks (
 					Block is near ground level if night-time mesh
 					differs from day-time mesh.
 				*/
-				if(d >= d_opt)
-				{
-					if(block->getDayNightDiff() == false)
+				if (d >= d_opt) {
+					if (!block->getIsUnderground() && !block->getDayNightDiff())
 						continue;
 				}
 
-				if (occ_cull && !block_is_invalid &&
+				if (m_occ_cull && !block_is_invalid &&
 						env->getMap().isBlockOccluded(block, cam_pos_nodes)) {
 					continue;
 				}
@@ -313,8 +346,7 @@ void RemoteClient::GetNextBlocks (
 				If block has been marked to not exist on disk (dummy)
 				and generating new ones is not wanted, skip block.
 			*/
-			if(generate == false && surely_not_found_on_disk == true)
-			{
+			if (!generate && surely_not_found_on_disk) {
 				// get next one.
 				continue;
 			}
@@ -322,8 +354,7 @@ void RemoteClient::GetNextBlocks (
 			/*
 				Add inexistent block to emerge queue.
 			*/
-			if(block == NULL || surely_not_found_on_disk || block_is_invalid)
-			{
+			if (block == NULL || surely_not_found_on_disk || block_is_invalid) {
 				if (emerge->enqueueBlockEmerge(peer_id, p, generate)) {
 					if (nearest_emerged_d == -1)
 						nearest_emerged_d = d;
@@ -337,7 +368,7 @@ void RemoteClient::GetNextBlocks (
 				continue;
 			}
 
-			if(nearest_sent_d == -1)
+			if (nearest_sent_d == -1)
 				nearest_sent_d = d;
 
 			/*
@@ -354,23 +385,23 @@ queue_full_break:
 
 	// If nothing was found for sending and nothing was queued for
 	// emerging, continue next time browsing from here
-	if(nearest_emerged_d != -1){
+	if (nearest_emerged_d != -1) {
 		new_nearest_unsent_d = nearest_emerged_d;
-	} else if(nearest_emergefull_d != -1){
+	} else if (nearest_emergefull_d != -1) {
 		new_nearest_unsent_d = nearest_emergefull_d;
 	} else {
-		if(d > full_d_max){
+		if (d > full_d_max) {
 			new_nearest_unsent_d = 0;
-			m_nothing_to_send_pause_timer = 2.0;
+			m_nothing_to_send_pause_timer = 2.0f;
 		} else {
-			if(nearest_sent_d != -1)
+			if (nearest_sent_d != -1)
 				new_nearest_unsent_d = nearest_sent_d;
 			else
 				new_nearest_unsent_d = d;
 		}
 	}
 
-	if(new_nearest_unsent_d != -1)
+	if (new_nearest_unsent_d != -1)
 		m_nearest_unsent_d = new_nearest_unsent_d;
 }
 
@@ -391,8 +422,8 @@ void RemoteClient::SentBlock(v3s16 p)
 	if (m_blocks_modified.find(p) != m_blocks_modified.end())
 		m_blocks_modified.erase(p);
 
-	if(m_blocks_sending.find(p) == m_blocks_sending.end())
-		m_blocks_sending[p] = 0.0;
+	if (m_blocks_sending.find(p) == m_blocks_sending.end())
+		m_blocks_sending[p] = 0.0f;
 	else
 		infostream<<"RemoteClient::SentBlock(): Sent block"
 				" already in m_blocks_sending"<<std::endl;
@@ -403,9 +434,9 @@ void RemoteClient::SetBlockNotSent(v3s16 p)
 	m_nearest_unsent_d = 0;
 	m_nothing_to_send_pause_timer = 0;
 
-	if(m_blocks_sending.find(p) != m_blocks_sending.end())
+	if (m_blocks_sending.find(p) != m_blocks_sending.end())
 		m_blocks_sending.erase(p);
-	if(m_blocks_sent.find(p) != m_blocks_sent.end())
+	if (m_blocks_sent.find(p) != m_blocks_sent.end())
 		m_blocks_sent.erase(p);
 	m_blocks_modified.insert(p);
 }
@@ -415,16 +446,13 @@ void RemoteClient::SetBlocksNotSent(std::map<v3s16, MapBlock*> &blocks)
 	m_nearest_unsent_d = 0;
 	m_nothing_to_send_pause_timer = 0;
 
-	for(std::map<v3s16, MapBlock*>::iterator
-			i = blocks.begin();
-			i != blocks.end(); ++i)
-	{
-		v3s16 p = i->first;
+	for (auto &block : blocks) {
+		v3s16 p = block.first;
 		m_blocks_modified.insert(p);
 
-		if(m_blocks_sending.find(p) != m_blocks_sending.end())
+		if (m_blocks_sending.find(p) != m_blocks_sending.end())
 			m_blocks_sending.erase(p);
-		if(m_blocks_sent.find(p) != m_blocks_sent.end())
+		if (m_blocks_sent.find(p) != m_blocks_sent.end())
 			m_blocks_sent.erase(p);
 	}
 }
@@ -465,8 +493,8 @@ void RemoteClient::notifyEvent(ClientStateEvent event)
 		{
 		case CSE_AuthAccept:
 			m_state = CS_AwaitingInit2;
-			if ((chosen_mech == AUTH_MECHANISM_SRP)
-					|| (chosen_mech == AUTH_MECHANISM_LEGACY_PASSWORD))
+			if (chosen_mech == AUTH_MECHANISM_SRP ||
+					chosen_mech == AUTH_MECHANISM_LEGACY_PASSWORD)
 				srp_verifier_delete((SRPVerifier *) auth_data);
 			chosen_mech = AUTH_MECHANISM_NONE;
 			break;
@@ -475,8 +503,8 @@ void RemoteClient::notifyEvent(ClientStateEvent event)
 			break;
 		case CSE_SetDenied:
 			m_state = CS_Denied;
-			if ((chosen_mech == AUTH_MECHANISM_SRP)
-					|| (chosen_mech == AUTH_MECHANISM_LEGACY_PASSWORD))
+			if (chosen_mech == AUTH_MECHANISM_SRP ||
+					chosen_mech == AUTH_MECHANISM_LEGACY_PASSWORD)
 				srp_verifier_delete((SRPVerifier *) auth_data);
 			chosen_mech = AUTH_MECHANISM_NONE;
 			break;
@@ -554,8 +582,7 @@ void RemoteClient::notifyEvent(ClientStateEvent event)
 			break;
 		case CSE_SudoSuccess:
 			m_state = CS_SudoMode;
-			if ((chosen_mech == AUTH_MECHANISM_SRP)
-					|| (chosen_mech == AUTH_MECHANISM_LEGACY_PASSWORD))
+			if (chosen_mech == AUTH_MECHANISM_SRP)
 				srp_verifier_delete((SRPVerifier *) auth_data);
 			chosen_mech = AUTH_MECHANISM_NONE;
 			break;
@@ -595,11 +622,11 @@ u64 RemoteClient::uptime() const
 	return porting::getTimeS() - m_connection_time;
 }
 
-ClientInterface::ClientInterface(con::Connection* con)
+ClientInterface::ClientInterface(const std::shared_ptr<con::Connection> & con)
 :
 	m_con(con),
 	m_env(NULL),
-	m_print_info_timer(0.0)
+	m_print_info_timer(0.0f)
 {
 
 }
@@ -609,30 +636,37 @@ ClientInterface::~ClientInterface()
 		Delete clients
 	*/
 	{
-		MutexAutoLock clientslock(m_clients_mutex);
+		RecursiveMutexAutoLock clientslock(m_clients_mutex);
 
-		for (UNORDERED_MAP<u16, RemoteClient*>::iterator i = m_clients.begin();
-			i != m_clients.end(); ++i) {
+		for (auto &client_it : m_clients) {
 			// Delete client
-			delete i->second;
+			delete client_it.second;
 		}
 	}
 }
 
-std::vector<u16> ClientInterface::getClientIDs(ClientState min_state)
+std::vector<session_t> ClientInterface::getClientIDs(ClientState min_state)
 {
-	std::vector<u16> reply;
-	MutexAutoLock clientslock(m_clients_mutex);
+	std::vector<session_t> reply;
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
 
-	for(UNORDERED_MAP<u16, RemoteClient*>::iterator i = m_clients.begin();
-		i != m_clients.end(); ++i) {
-		if (i->second->getState() >= min_state)
-			reply.push_back(i->second->peer_id);
+	for (const auto &m_client : m_clients) {
+		if (m_client.second->getState() >= min_state)
+			reply.push_back(m_client.second->peer_id);
 	}
 
 	return reply;
 }
 
+void ClientInterface::markBlockposAsNotSent(const v3s16 &pos)
+{
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
+	for (const auto &client : m_clients) {
+		if (client.second->getState() >= CS_Active)
+			client.second->SetBlockNotSent(pos);
+	}
+}
+
 /**
  * Verify if user limit was reached.
  * User limit count all clients from HelloSent state (MT protocol user) to Active state
@@ -646,25 +680,24 @@ bool ClientInterface::isUserLimitReached()
 void ClientInterface::step(float dtime)
 {
 	m_print_info_timer += dtime;
-	if(m_print_info_timer >= 30.0)
-	{
-		m_print_info_timer = 0.0;
+	if (m_print_info_timer >= 30.0f) {
+		m_print_info_timer = 0.0f;
 		UpdatePlayerList();
 	}
 }
 
 void ClientInterface::UpdatePlayerList()
 {
-	if (m_env != NULL) {
-		std::vector<u16> clients = getClientIDs();
+	if (m_env) {
+		std::vector<session_t> clients = getClientIDs();
 		m_clients_names.clear();
 
 
-		if(!clients.empty())
+		if (!clients.empty())
 			infostream<<"Players:"<<std::endl;
 
-		for (std::vector<u16>::iterator i = clients.begin(); i != clients.end(); ++i) {
-			RemotePlayer *player = m_env->getPlayer(*i);
+		for (session_t i : clients) {
+			RemotePlayer *player = m_env->getPlayer(i);
 
 			if (player == NULL)
 				continue;
@@ -672,29 +705,28 @@ void ClientInterface::UpdatePlayerList()
 			infostream << "* " << player->getName() << "\t";
 
 			{
-				MutexAutoLock clientslock(m_clients_mutex);
-				RemoteClient* client = lockedGetClientNoEx(*i);
-				if(client != NULL)
+				RecursiveMutexAutoLock clientslock(m_clients_mutex);
+				RemoteClient* client = lockedGetClientNoEx(i);
+				if (client)
 					client->PrintInfo(infostream);
 			}
 
-			m_clients_names.push_back(player->getName());
+			m_clients_names.emplace_back(player->getName());
 		}
 	}
 }
 
-void ClientInterface::send(u16 peer_id, u8 channelnum,
-		NetworkPacket* pkt, bool reliable)
+void ClientInterface::send(session_t peer_id, u8 channelnum,
+		NetworkPacket *pkt, bool reliable)
 {
 	m_con->Send(peer_id, channelnum, pkt, reliable);
 }
 
 void ClientInterface::sendToAll(NetworkPacket *pkt)
 {
-	MutexAutoLock clientslock(m_clients_mutex);
-	for (UNORDERED_MAP<u16, RemoteClient*>::iterator i = m_clients.begin();
-			i != m_clients.end(); ++i) {
-		RemoteClient *client = i->second;
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
+	for (auto &client_it : m_clients) {
+		RemoteClient *client = client_it.second;
 
 		if (client->net_proto_version != 0) {
 			m_con->Send(client->peer_id,
@@ -704,10 +736,35 @@ void ClientInterface::sendToAll(NetworkPacket *pkt)
 	}
 }
 
-RemoteClient* ClientInterface::getClientNoEx(u16 peer_id, ClientState state_min)
+void ClientInterface::sendToAllCompat(NetworkPacket *pkt, NetworkPacket *legacypkt,
+		u16 min_proto_ver)
 {
-	MutexAutoLock clientslock(m_clients_mutex);
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
+	for (auto &client_it : m_clients) {
+		RemoteClient *client = client_it.second;
+		NetworkPacket *pkt_to_send = nullptr;
+
+		if (client->net_proto_version >= min_proto_ver) {
+			pkt_to_send = pkt;
+		} else if (client->net_proto_version != 0) {
+			pkt_to_send = legacypkt;
+		} else {
+			warningstream << "Client with unhandled version to handle: '"
+				<< client->net_proto_version << "'";
+			continue;
+		}
+
+		m_con->Send(client->peer_id,
+			clientCommandFactoryTable[pkt_to_send->getCommand()].channel,
+			pkt_to_send,
+			clientCommandFactoryTable[pkt_to_send->getCommand()].reliable);
+	}
+}
+
+RemoteClient* ClientInterface::getClientNoEx(session_t peer_id, ClientState state_min)
+{
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
+	RemoteClientMap::const_iterator n = m_clients.find(peer_id);
 	// The client may not exist; clients are immediately removed if their
 	// access is denied, and this event occurs later then.
 	if (n == m_clients.end())
@@ -715,13 +772,13 @@ RemoteClient* ClientInterface::getClientNoEx(u16 peer_id, ClientState state_min)
 
 	if (n->second->getState() >= state_min)
 		return n->second;
-	else
-		return NULL;
+
+	return NULL;
 }
 
-RemoteClient* ClientInterface::lockedGetClientNoEx(u16 peer_id, ClientState state_min)
+RemoteClient* ClientInterface::lockedGetClientNoEx(session_t peer_id, ClientState state_min)
 {
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RemoteClientMap::const_iterator n = m_clients.find(peer_id);
 	// The client may not exist; clients are immediately removed if their
 	// access is denied, and this event occurs later then.
 	if (n == m_clients.end())
@@ -729,14 +786,14 @@ RemoteClient* ClientInterface::lockedGetClientNoEx(u16 peer_id, ClientState stat
 
 	if (n->second->getState() >= state_min)
 		return n->second;
-	else
-		return NULL;
+
+	return NULL;
 }
 
-ClientState ClientInterface::getClientState(u16 peer_id)
+ClientState ClientInterface::getClientState(session_t peer_id)
 {
-	MutexAutoLock clientslock(m_clients_mutex);
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
+	RemoteClientMap::const_iterator n = m_clients.find(peer_id);
 	// The client may not exist; clients are immediately removed if their
 	// access is denied, and this event occurs later then.
 	if (n == m_clients.end())
@@ -745,22 +802,22 @@ ClientState ClientInterface::getClientState(u16 peer_id)
 	return n->second->getState();
 }
 
-void ClientInterface::setPlayerName(u16 peer_id,std::string name)
+void ClientInterface::setPlayerName(session_t peer_id, const std::string &name)
 {
-	MutexAutoLock clientslock(m_clients_mutex);
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RecursiveMutexAutoLock clientslock(m_clients_mutex);
+	RemoteClientMap::iterator n = m_clients.find(peer_id);
 	// The client may not exist; clients are immediately removed if their
 	// access is denied, and this event occurs later then.
 	if (n != m_clients.end())
 		n->second->setName(name);
 }
 
-void ClientInterface::DeleteClient(u16 peer_id)
+void ClientInterface::DeleteClient(session_t peer_id)
 {
-	MutexAutoLock conlock(m_clients_mutex);
+	RecursiveMutexAutoLock conlock(m_clients_mutex);
 
 	// Error check
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RemoteClientMap::iterator n = m_clients.find(peer_id);
 	// The client may not exist; clients are immediately removed if their
 	// access is denied, and this event occurs later then.
 	if (n == m_clients.end())
@@ -772,12 +829,8 @@ void ClientInterface::DeleteClient(u16 peer_id)
 	//TODO this should be done by client destructor!!!
 	RemoteClient *client = n->second;
 	// Handle objects
-	for(std::set<u16>::iterator
-			i = client->m_known_objects.begin();
-			i != client->m_known_objects.end(); ++i)
-	{
+	for (u16 id : client->m_known_objects) {
 		// Get object
-		u16 id = *i;
 		ServerActiveObject* obj = m_env->getActiveObject(id);
 
 		if(obj && obj->m_known_by_count > 0)
@@ -789,12 +842,12 @@ void ClientInterface::DeleteClient(u16 peer_id)
 	m_clients.erase(peer_id);
 }
 
-void ClientInterface::CreateClient(u16 peer_id)
+void ClientInterface::CreateClient(session_t peer_id)
 {
-	MutexAutoLock conlock(m_clients_mutex);
+	RecursiveMutexAutoLock conlock(m_clients_mutex);
 
 	// Error check
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RemoteClientMap::iterator n = m_clients.find(peer_id);
 	// The client shouldn't already exist
 	if (n != m_clients.end()) return;
 
@@ -804,13 +857,13 @@ void ClientInterface::CreateClient(u16 peer_id)
 	m_clients[client->peer_id] = client;
 }
 
-void ClientInterface::event(u16 peer_id, ClientStateEvent event)
+void ClientInterface::event(session_t peer_id, ClientStateEvent event)
 {
 	{
-		MutexAutoLock clientlock(m_clients_mutex);
+		RecursiveMutexAutoLock clientlock(m_clients_mutex);
 
 		// Error check
-		UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+		RemoteClientMap::iterator n = m_clients.find(peer_id);
 
 		// No client to deliver event
 		if (n == m_clients.end())
@@ -826,12 +879,12 @@ void ClientInterface::event(u16 peer_id, ClientStateEvent event)
 	}
 }
 
-u16 ClientInterface::getProtocolVersion(u16 peer_id)
+u16 ClientInterface::getProtocolVersion(session_t peer_id)
 {
-	MutexAutoLock conlock(m_clients_mutex);
+	RecursiveMutexAutoLock conlock(m_clients_mutex);
 
 	// Error check
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RemoteClientMap::iterator n = m_clients.find(peer_id);
 
 	// No client to get version
 	if (n == m_clients.end())
@@ -840,16 +893,17 @@ u16 ClientInterface::getProtocolVersion(u16 peer_id)
 	return n->second->net_proto_version;
 }
 
-void ClientInterface::setClientVersion(u16 peer_id, u8 major, u8 minor, u8 patch, std::string full)
+void ClientInterface::setClientVersion(session_t peer_id, u8 major, u8 minor, u8 patch,
+		const std::string &full)
 {
-	MutexAutoLock conlock(m_clients_mutex);
+	RecursiveMutexAutoLock conlock(m_clients_mutex);
 
 	// Error check
-	UNORDERED_MAP<u16, RemoteClient*>::iterator n = m_clients.find(peer_id);
+	RemoteClientMap::iterator n = m_clients.find(peer_id);
 
 	// No client to set versions
 	if (n == m_clients.end())
 		return;
 
-	n->second->setVersionInfo(major,minor,patch,full);
+	n->second->setVersionInfo(major, minor, patch, full);
 }
