@@ -501,7 +501,7 @@ private:
 
 	// Connection
 	std::shared_ptr<con::Connection> m_con;
-	std::mutex m_clients_mutex;
+	std::recursive_mutex m_clients_mutex;
 	// Connected clients (behind the con mutex)
 	RemoteClientMap m_clients;
 	std::vector<std::string> m_clients_names; //for announcing masterserver
