@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef CLOUDPARAMS_HEADER
-#define CLOUDPARAMS_HEADER
+#pragma once
 
 struct CloudParams
 {
@@ -29,5 +28,3 @@ struct CloudParams
 	float height;
 	v2f speed;
 };
-
-#endif
