@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef CONSTANTS_HEADER
-#define CONSTANTS_HEADER
+#pragma once
 
 /*
 	All kinds of constants.
@@ -72,7 +71,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 // floating-point and integer positions, which potentially give wrong
 // results. (negative coordinates, values between nodes, ...)
 // Use floatToInt(p, BS) and intToFloat(p, BS).
-#define BS (10.0)
+#define BS 10.0f
 
 // Dimension of a MapBlock
 #define MAP_BLOCKSIZE 16
@@ -80,6 +79,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 // the main loop (related to TempMods and day/night)
 //#define MAP_BLOCKSIZE 32
 
+// Player step height in nodes
+#define PLAYER_DEFAULT_STEPHEIGHT 0.6f
+
 /*
     Old stuff that shouldn't be hardcoded
 */
@@ -87,11 +89,11 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 // Size of player's main inventory
 #define PLAYER_INVENTORY_SIZE (8 * 4)
 
-// Maximum hit points of a player
-#define PLAYER_MAX_HP 20
+// Default maximum hit points of a player
+#define PLAYER_MAX_HP_DEFAULT 20
 
-// Maximal breath of a player
-#define PLAYER_MAX_BREATH 11
+// Default maximal breath of a player
+#define PLAYER_MAX_BREATH_DEFAULT 11
 
 // Number of different files to try to save a player to if the first fails
 // (because of a case-insensitive filesystem)
@@ -115,5 +117,3 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define TTF_DEFAULT_FONT_SIZE (16)
 #endif
 #define DEFAULT_FONT_SIZE (10)
-
-#endif
