@@ -80,6 +80,9 @@ struct CraftInput
 		method(method_), width(width_), items(items_)
 	{}
 
+	// Returns true if all items are empty.
+	bool empty() const;
+
 	std::string dump() const;
 };
 
@@ -132,6 +135,23 @@ struct CraftReplacements
 class CraftDefinition
 {
 public:
+	/*
+		Craft recipe priorities, from low to high
+
+		Recipes are searched from latest to first.
+		If a recipe with higher priority than a previous found one is
+		encountered, it is selected instead.
+	*/
+	enum RecipePriority
+	{
+		PRIORITY_NO_RECIPE,
+		PRIORITY_TOOLREPAIR,
+		PRIORITY_SHAPELESS_AND_GROUPS,
+		PRIORITY_SHAPELESS,
+		PRIORITY_SHAPED_AND_GROUPS,
+		PRIORITY_SHAPED,
+	};
+
 	CraftDefinition() = default;
 	virtual ~CraftDefinition() = default;
 
@@ -140,6 +160,10 @@ public:
 
 	// Checks whether the recipe is applicable
 	virtual bool check(const CraftInput &input, IGameDef *gamedef) const=0;
+	RecipePriority getPriority() const
+	{
+		return priority;
+	}
 	// Returns the output structure, meaning depends on crafting method
 	// The implementation can assume that check(input) returns true
 	virtual CraftOutput getOutput(const CraftInput &input, IGameDef *gamedef) const=0;
@@ -149,13 +173,20 @@ public:
 	virtual void decrementInput(CraftInput &input,
 		std::vector<ItemStack> &output_replacements, IGameDef *gamedef) const=0;
 
-	virtual CraftHashType getHashType() const = 0;
+	CraftHashType getHashType() const
+	{
+		return hash_type;
+	}
 	virtual u64 getHash(CraftHashType type) const = 0;
 
 	// to be called after all mods are loaded, so that we catch all aliases
 	virtual void initHash(IGameDef *gamedef) = 0;
 
 	virtual std::string dump() const=0;
+
+protected:
+	CraftHashType hash_type;
+	RecipePriority priority;
 };
 
 /*
@@ -168,15 +199,12 @@ class CraftDefinitionShaped: public CraftDefinition
 {
 public:
 	CraftDefinitionShaped() = delete;
-
 	CraftDefinitionShaped(
-			const std::string &output_,
-			unsigned int width_,
-			const std::vector<std::string> &recipe_,
-			const CraftReplacements &replacements_):
-		output(output_), width(width_), recipe(recipe_),
-		replacements(replacements_)
-	{}
+		const std::string &output_,
+		unsigned int width_,
+		const std::vector<std::string> &recipe_,
+		const CraftReplacements &replacements_);
+
 	virtual ~CraftDefinitionShaped() = default;
 
 	virtual std::string getName() const;
@@ -186,7 +214,6 @@ public:
 	virtual void decrementInput(CraftInput &input,
 		std::vector<ItemStack> &output_replacements, IGameDef *gamedef) const;
 
-	virtual CraftHashType getHashType() const;
 	virtual u64 getHash(CraftHashType type) const;
 
 	virtual void initHash(IGameDef *gamedef);
@@ -218,11 +245,10 @@ class CraftDefinitionShapeless: public CraftDefinition
 public:
 	CraftDefinitionShapeless() = delete;
 	CraftDefinitionShapeless(
-			const std::string &output_,
-			const std::vector<std::string> &recipe_,
-			const CraftReplacements &replacements_):
-		output(output_), recipe(recipe_), replacements(replacements_)
-	{}
+		const std::string &output_,
+		const std::vector<std::string> &recipe_,
+		const CraftReplacements &replacements_);
+
 	virtual ~CraftDefinitionShapeless() = default;
 
 	virtual std::string getName() const;
@@ -232,7 +258,6 @@ public:
 	virtual void decrementInput(CraftInput &input,
 		std::vector<ItemStack> &output_replacements, IGameDef *gamedef) const;
 
-	virtual CraftHashType getHashType() const;
 	virtual u64 getHash(CraftHashType type) const;
 
 	virtual void initHash(IGameDef *gamedef);
@@ -262,9 +287,8 @@ class CraftDefinitionToolRepair: public CraftDefinition
 {
 public:
 	CraftDefinitionToolRepair() = delete;
-	CraftDefinitionToolRepair(float additional_wear_):
-		additional_wear(additional_wear_)
-	{}
+	CraftDefinitionToolRepair(float additional_wear_);
+
 	virtual ~CraftDefinitionToolRepair() = default;
 
 	virtual std::string getName() const;
@@ -274,10 +298,12 @@ public:
 	virtual void decrementInput(CraftInput &input,
 		std::vector<ItemStack> &output_replacements, IGameDef *gamedef) const;
 
-	virtual CraftHashType getHashType() const { return CRAFT_HASH_TYPE_COUNT; }
 	virtual u64 getHash(CraftHashType type) const { return 2; }
 
-	virtual void initHash(IGameDef *gamedef) {}
+	virtual void initHash(IGameDef *gamedef)
+	{
+		hash_type = CRAFT_HASH_TYPE_COUNT;
+	}
 
 	virtual std::string dump() const;
 
@@ -299,12 +325,11 @@ class CraftDefinitionCooking: public CraftDefinition
 public:
 	CraftDefinitionCooking() = delete;
 	CraftDefinitionCooking(
-			const std::string &output_,
-			const std::string &recipe_,
-			float cooktime_,
-			const CraftReplacements &replacements_):
-		output(output_), recipe(recipe_), cooktime(cooktime_), replacements(replacements_)
-	{}
+		const std::string &output_,
+		const std::string &recipe_,
+		float cooktime_,
+		const CraftReplacements &replacements_);
+
 	virtual ~CraftDefinitionCooking() = default;
 
 	virtual std::string getName() const;
@@ -314,7 +339,6 @@ public:
 	virtual void decrementInput(CraftInput &input,
 		std::vector<ItemStack> &output_replacements, IGameDef *gamedef) const;
 
-	virtual CraftHashType getHashType() const;
 	virtual u64 getHash(CraftHashType type) const;
 
 	virtual void initHash(IGameDef *gamedef);
@@ -344,11 +368,11 @@ class CraftDefinitionFuel: public CraftDefinition
 {
 public:
 	CraftDefinitionFuel() = delete;
-	CraftDefinitionFuel(const std::string &recipe_,
-			float burntime_,
-			const CraftReplacements &replacements_):
-		recipe(recipe_), burntime(burntime_), replacements(replacements_)
-	{}
+	CraftDefinitionFuel(
+		const std::string &recipe_,
+		float burntime_,
+		const CraftReplacements &replacements_);
+
 	virtual ~CraftDefinitionFuel() = default;
 
 	virtual std::string getName() const;
@@ -358,7 +382,6 @@ public:
 	virtual void decrementInput(CraftInput &input,
 		std::vector<ItemStack> &output_replacements, IGameDef *gamedef) const;
 
-	virtual CraftHashType getHashType() const;
 	virtual u64 getHash(CraftHashType type) const;
 
 	virtual void initHash(IGameDef *gamedef);
@@ -411,9 +434,8 @@ public:
 	virtual std::vector<CraftDefinition*> getCraftRecipes(CraftOutput &output,
 			IGameDef *gamedef, unsigned limit=0) const=0;
 
-	virtual bool clearCraftRecipesByOutput(const CraftOutput &output, IGameDef *gamedef) = 0;
-	virtual bool clearCraftRecipesByInput(CraftMethod craft_method,
-			unsigned int craft_grid_width, const std::vector<std::string> &recipe, IGameDef *gamedef) = 0;
+	virtual bool clearCraftsByOutput(const CraftOutput &output, IGameDef *gamedef) = 0;
+	virtual bool clearCraftsByInput(const CraftInput &input, IGameDef *gamedef) = 0;
 
 	// Print crafting recipes for debugging
 	virtual std::string dump() const=0;
