@@ -17,15 +17,14 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef FACE_POSITION_CACHE_HEADER
-#define FACE_POSITION_CACHE_HEADER
+#pragma once
 
 #include "irr_v3d.h"
-#include "threading/mutex.h"
-#include "util/cpp11_container.h"
 
 #include <map>
 #include <vector>
+#include <unordered_map>
+#include <mutex>
 
 /*
  * This class permits caching getFacePosition call results.
@@ -37,8 +36,6 @@ public:
 
 private:
 	static const std::vector<v3s16> &generateFacePosition(u16 d);
-	static UNORDERED_MAP<u16, std::vector<v3s16> > cache;
-	static Mutex cache_mutex;
+	static std::unordered_map<u16, std::vector<v3s16>> cache;
+	static std::mutex cache_mutex;
 };
-
-#endif
