@@ -31,9 +31,6 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 //#define DGEN_USE_TORCHES
 
-NoiseParams nparams_dungeon_density(0.9, 0.5, v3f(500.0, 500.0, 500.0), 0, 2, 0.8, 2.0);
-NoiseParams nparams_dungeon_alt_wall(-0.4, 1.0, v3f(40.0, 40.0, 40.0), 32474, 6, 1.1, 2.0);
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -69,28 +66,26 @@ DungeonGen::DungeonGen(const NodeDefManager *ndef,
 		dp.room_size_max       = v3s16(8, 6, 8);
 		dp.room_size_large_min = v3s16(8, 8, 8);
 		dp.room_size_large_max = v3s16(16, 16, 16);
-		dp.rooms_min           = 2;
-		dp.rooms_max           = 16;
+		dp.large_room_chance   = 1;
+		dp.num_rooms           = 8;
+		dp.num_dungeons        = 1;
 		dp.notifytype          = GENNOTIFY_DUNGEON;
 
-		dp.np_density  = nparams_dungeon_density;
-		dp.np_alt_wall = nparams_dungeon_alt_wall;
+		dp.np_alt_wall = 
+			NoiseParams(-0.4, 1.0, v3f(40.0, 40.0, 40.0), 32474, 6, 1.1, 2.0);
 	}
 }
 
 
 void DungeonGen::generate(MMVManip *vm, u32 bseed, v3s16 nmin, v3s16 nmax)
 {
+	if (dp.num_dungeons == 0)
+		return;
+
 	assert(vm);
 
 	//TimeTaker t("gen dungeons");
 
-	float nval_density = NoisePerlin3D(&dp.np_density, nmin.X, nmin.Y, nmin.Z, dp.seed);
-	if (nval_density < 1.0f)
-		return;
-
-	static const bool preserve_ignore = !g_settings->getBool("projecting_dungeons");
-
 	this->vm = vm;
 	this->blockseed = bseed;
 	random.seed(bseed + 2);
@@ -99,9 +94,13 @@ void DungeonGen::generate(MMVManip *vm, u32 bseed, v3s16 nmin, v3s16 nmax)
 	vm->clearFlag(VMANIP_FLAG_DUNGEON_INSIDE | VMANIP_FLAG_DUNGEON_PRESERVE);
 
 	if (dp.only_in_ground) {
-		// Set all air and liquid drawtypes to be untouchable to make dungeons
-		// open to air and liquids. Optionally set ignore to be untouchable to
-		// prevent projecting dungeons.
+		// Set all air and liquid drawtypes to be untouchable to make dungeons generate
+		// in ground only.
+		// Set 'ignore' to be untouchable to prevent generation in ungenerated neighbor
+		// mapchunks, to avoid dungeon rooms generating outside ground.
+		// Like randomwalk caves, preserve nodes that have 'is_ground_content = false',
+		// to avoid dungeons that generate out beyond the edge of a mapchunk destroying
+		// nodes added by mods in 'register_on_generated()'.
 		for (s16 z = nmin.Z; z <= nmax.Z; z++) {
 			for (s16 y = nmin.Y; y <= nmax.Y; y++) {
 				u32 i = vm->m_area.index(nmin.X, y, z);
@@ -109,7 +108,7 @@ void DungeonGen::generate(MMVManip *vm, u32 bseed, v3s16 nmin, v3s16 nmax)
 					content_t c = vm->m_data[i].getContent();
 					NodeDrawType dtype = ndef->get(c).drawtype;
 					if (dtype == NDT_AIRLIKE || dtype == NDT_LIQUID ||
-							(preserve_ignore && c == CONTENT_IGNORE))
+							c == CONTENT_IGNORE || !ndef->get(c).is_ground_content)
 						vm->m_flags[i] |= VMANIP_FLAG_DUNGEON_PRESERVE;
 					i++;
 				}
@@ -118,7 +117,7 @@ void DungeonGen::generate(MMVManip *vm, u32 bseed, v3s16 nmin, v3s16 nmax)
 	}
 
 	// Add them
-	for (u32 i = 0; i < std::floor(nval_density); i++)
+	for (u32 i = 0; i < dp.num_dungeons; i++)
 		makeDungeon(v3s16(1, 1, 1) * MAP_BLOCKSIZE);
 
 	// Optionally convert some structure to alternative structure
@@ -149,19 +148,13 @@ void DungeonGen::makeDungeon(v3s16 start_padding)
 
 	/*
 		Find place for first room.
-		There is a 1 in 4 chance of the first room being 'large',
-		all other rooms are not 'large'.
 	*/
 	bool fits = false;
 	for (u32 i = 0; i < 100 && !fits; i++) {
-		bool is_large_room = ((random.next() & 3) == 1);
-		if (is_large_room) {
-			roomsize.Z = random.range(
-				dp.room_size_large_min.Z, dp.room_size_large_max.Z);
-			roomsize.Y = random.range(
-				dp.room_size_large_min.Y, dp.room_size_large_max.Y);
-			roomsize.X = random.range(
-				dp.room_size_large_min.X, dp.room_size_large_max.X);
+		if (dp.large_room_chance >= 1) {
+			roomsize.Z = random.range(dp.room_size_large_min.Z, dp.room_size_large_max.Z);
+			roomsize.Y = random.range(dp.room_size_large_min.Y, dp.room_size_large_max.Y);
+			roomsize.X = random.range(dp.room_size_large_min.X, dp.room_size_large_max.X);
 		} else {
 			roomsize.Z = random.range(dp.room_size_min.Z, dp.room_size_max.Z);
 			roomsize.Y = random.range(dp.room_size_min.Y, dp.room_size_max.Y);
@@ -203,8 +196,7 @@ void DungeonGen::makeDungeon(v3s16 start_padding)
 	*/
 	v3s16 last_room_center = roomplace + v3s16(roomsize.X / 2, 1, roomsize.Z / 2);
 
-	u32 room_count = random.range(dp.rooms_min, dp.rooms_max);
-	for (u32 i = 0; i < room_count; i++) {
+	for (u32 i = 0; i < dp.num_rooms; i++) {
 		// Make a room to the determined place
 		makeRoom(roomsize, roomplace);
 
@@ -218,7 +210,7 @@ void DungeonGen::makeDungeon(v3s16 start_padding)
 #endif
 
 		// Quit if last room
-		if (i == room_count - 1)
+		if (i + 1 == dp.num_rooms)
 			break;
 
 		// Determine walker start position
@@ -256,9 +248,16 @@ void DungeonGen::makeDungeon(v3s16 start_padding)
 		makeCorridor(doorplace, doordir, corridor_end, corridor_end_dir);
 
 		// Find a place for a random sized room
-		roomsize.Z = random.range(dp.room_size_min.Z, dp.room_size_max.Z);
-		roomsize.Y = random.range(dp.room_size_min.Y, dp.room_size_max.Y);
-		roomsize.X = random.range(dp.room_size_min.X, dp.room_size_max.X);
+		if (dp.large_room_chance > 1 && random.range(1, dp.large_room_chance) == 1) {
+			// Large room
+			roomsize.Z = random.range(dp.room_size_large_min.Z, dp.room_size_large_max.Z);
+			roomsize.Y = random.range(dp.room_size_large_min.Y, dp.room_size_large_max.Y);
+			roomsize.X = random.range(dp.room_size_large_min.X, dp.room_size_large_max.X);
+		} else {
+			roomsize.Z = random.range(dp.room_size_min.Z, dp.room_size_max.Z);
+			roomsize.Y = random.range(dp.room_size_min.Y, dp.room_size_max.Y);
+			roomsize.X = random.range(dp.room_size_min.X, dp.room_size_max.X);
+		}
 
 		m_pos = corridor_end;
 		m_dir = corridor_end_dir;
@@ -271,7 +270,6 @@ void DungeonGen::makeDungeon(v3s16 start_padding)
 		else
 			// Don't actually make a door
 			roomplace -= doordir;
-
 	}
 }
 
@@ -490,7 +488,7 @@ void DungeonGen::makeCorridor(v3s16 doorplace, v3s16 doordir,
 		if (partcount >= partlength) {
 			partcount = 0;
 
-			dir = random_turn(random, dir);
+			random_turn(random, dir);
 
 			partlength = random.range(1, length);
 
@@ -651,20 +649,19 @@ v3s16 turn_xz(v3s16 olddir, int t)
 }
 
 
-v3s16 random_turn(PseudoRandom &random, v3s16 olddir)
+void random_turn(PseudoRandom &random, v3s16 &dir)
 {
 	int turn = random.range(0, 2);
-	v3s16 dir;
-	if (turn == 0)
-		// Go straight
-		dir = olddir;
-	else if (turn == 1)
+	if (turn == 0) {
+		// Go straight: nothing to do
+		return;
+	} else if (turn == 1) {
 		// Turn right
-		dir = turn_xz(olddir, 0);
-	else
+		dir = turn_xz(dir, 0);
+	} else {
 		// Turn left
-		dir = turn_xz(olddir, 1);
-	return dir;
+		dir = turn_xz(dir, 1);
+	}
 }
 
 
