@@ -23,10 +23,10 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <math.h>
+#include <cmath>
 #include "noise.h"
 #include <iostream>
-#include <string.h> // memset
+#include <cstring> // memset
 #include "debug.h"
 #include "util/numeric.h"
 #include "util/string.h"
@@ -47,8 +47,8 @@ typedef float (*Interp3dFxn)(
 		float x, float y, float z);
 
 float cos_lookup[16] = {
-	1.0,  0.9238,  0.7071,  0.3826, 0, -0.3826, -0.7071, -0.9238,
-	1.0, -0.9238, -0.7071, -0.3826, 0,  0.3826,  0.7071,  0.9238
+	1.0f,  0.9238f,  0.7071f,  0.3826f, .0f, -0.3826f, -0.7071f, -0.9238f,
+	1.0f, -0.9238f, -0.7071f, -0.3826f, .0f,  0.3826f,  0.7071f,  0.9238f
 };
 
 FlagDesc flagdesc_noiseparams[] = {
@@ -262,8 +262,8 @@ float noise2d_gradient(float x, float y, s32 seed, bool eased)
 	// Interpolate
 	if (eased)
 		return biLinearInterpolation(v00, v10, v01, v11, xl, yl);
-	else
-		return biLinearInterpolationNoEase(v00, v10, v01, v11, xl, yl);
+
+	return biLinearInterpolationNoEase(v00, v10, v01, v11, xl, yl);
 }
 
 
@@ -292,12 +292,12 @@ float noise3d_gradient(float x, float y, float z, s32 seed, bool eased)
 			v000, v100, v010, v110,
 			v001, v101, v011, v111,
 			xl, yl, zl);
-	} else {
-		return triLinearInterpolationNoEase(
-			v000, v100, v010, v110,
-			v001, v101, v011, v111,
-			xl, yl, zl);
 	}
+
+	return triLinearInterpolationNoEase(
+		v000, v100, v010, v110,
+		v001, v101, v011, v111,
+		xl, yl, zl);
 }
 
 
@@ -324,7 +324,7 @@ float noise2d_perlin_abs(float x, float y, s32 seed,
 	float f = 1.0;
 	float g = 1.0;
 	for (int i = 0; i < octaves; i++) {
-		a += g * fabs(noise2d_gradient(x * f, y * f, seed + i, eased));
+		a += g * std::fabs(noise2d_gradient(x * f, y * f, seed + i, eased));
 		f *= 2.0;
 		g *= persistence;
 	}
@@ -354,7 +354,7 @@ float noise3d_perlin_abs(float x, float y, float z, s32 seed,
 	float f = 1.0;
 	float g = 1.0;
 	for (int i = 0; i < octaves; i++) {
-		a += g * fabs(noise3d_gradient(x * f, y * f, z * f, seed + i, eased));
+		a += g * std::fabs(noise3d_gradient(x * f, y * f, z * f, seed + i, eased));
 		f *= 2.0;
 		g *= persistence;
 	}
@@ -364,7 +364,7 @@ float noise3d_perlin_abs(float x, float y, float z, s32 seed,
 
 float contour(float v)
 {
-	v = fabs(v);
+	v = std::fabs(v);
 	if (v >= 1.0)
 		return 0.0;
 	return (1.0 - v);
@@ -389,7 +389,7 @@ float NoisePerlin2D(NoiseParams *np, float x, float y, s32 seed)
 			np->flags & (NOISE_FLAG_DEFAULTS | NOISE_FLAG_EASED));
 
 		if (np->flags & NOISE_FLAG_ABSVALUE)
-			noiseval = fabs(noiseval);
+			noiseval = std::fabs(noiseval);
 
 		a += g * noiseval;
 		f *= np->lacunarity;
@@ -416,7 +416,7 @@ float NoisePerlin3D(NoiseParams *np, float x, float y, float z, s32 seed)
 			np->flags & NOISE_FLAG_EASED);
 
 		if (np->flags & NOISE_FLAG_ABSVALUE)
-			noiseval = fabs(noiseval);
+			noiseval = std::fabs(noiseval);
 
 		a += g * noiseval;
 		f *= np->lacunarity;
@@ -435,10 +435,6 @@ Noise::Noise(NoiseParams *np_, s32 seed, u32 sx, u32 sy, u32 sz)
 	this->sy   = sy;
 	this->sz   = sz;
 
-	this->persist_buf  = NULL;
-	this->gradient_buf = NULL;
-	this->result       = NULL;
-
 	allocBuffers();
 }
 
@@ -526,9 +522,9 @@ void Noise::resizeNoiseBuf(bool is3d)
 
 	// + 2 for the two initial endpoints
 	// + 1 for potentially crossing a boundary due to offset
-	size_t nlx = (size_t)ceil(num_noise_points_x) + 3;
-	size_t nly = (size_t)ceil(num_noise_points_y) + 3;
-	size_t nlz = is3d ? (size_t)ceil(num_noise_points_z) + 3 : 1;
+	size_t nlx = (size_t)std::ceil(num_noise_points_x) + 3;
+	size_t nly = (size_t)std::ceil(num_noise_points_y) + 3;
+	size_t nlz = is3d ? (size_t)std::ceil(num_noise_points_z) + 3 : 1;
 
 	delete[] noise_buf;
 	try {
@@ -565,8 +561,8 @@ void Noise::gradientMap2D(
 	Interp2dFxn interpolate = eased ?
 		biLinearInterpolation : biLinearInterpolationNoEase;
 
-	x0 = floor(x);
-	y0 = floor(y);
+	x0 = std::floor(x);
+	y0 = std::floor(y);
 	u = x - (float)x0;
 	v = y - (float)y0;
 	orig_u = u;
@@ -630,9 +626,9 @@ void Noise::gradientMap3D(
 	Interp3dFxn interpolate = (np.flags & NOISE_FLAG_EASED) ?
 		triLinearInterpolation : triLinearInterpolationNoEase;
 
-	x0 = floor(x);
-	y0 = floor(y);
-	z0 = floor(z);
+	x0 = std::floor(x);
+	y0 = std::floor(y);
+	z0 = std::floor(z);
 	u = x - (float)x0;
 	v = y - (float)y0;
 	w = z - (float)z0;
@@ -734,7 +730,7 @@ float *Noise::perlinMap2D(float x, float y, float *persistence_map)
 		g *= np.persist;
 	}
 
-	if (fabs(np.offset - 0.f) > 0.00001 || fabs(np.scale - 1.f) > 0.00001) {
+	if (std::fabs(np.offset - 0.f) > 0.00001 || std::fabs(np.scale - 1.f) > 0.00001) {
 		for (size_t i = 0; i != bufsize; i++)
 			result[i] = result[i] * np.scale + np.offset;
 	}
@@ -772,7 +768,7 @@ float *Noise::perlinMap3D(float x, float y, float z, float *persistence_map)
 		g *= np.persist;
 	}
 
-	if (fabs(np.offset - 0.f) > 0.00001 || fabs(np.scale - 1.f) > 0.00001) {
+	if (std::fabs(np.offset - 0.f) > 0.00001 || std::fabs(np.scale - 1.f) > 0.00001) {
 		for (size_t i = 0; i != bufsize; i++)
 			result[i] = result[i] * np.scale + np.offset;
 	}
@@ -782,19 +778,19 @@ float *Noise::perlinMap3D(float x, float y, float z, float *persistence_map)
 
 
 void Noise::updateResults(float g, float *gmap,
-	float *persistence_map, size_t bufsize)
+	const float *persistence_map, size_t bufsize)
 {
 	// This looks very ugly, but it is 50-70% faster than having
 	// conditional statements inside the loop
 	if (np.flags & NOISE_FLAG_ABSVALUE) {
 		if (persistence_map) {
 			for (size_t i = 0; i != bufsize; i++) {
-				result[i] += gmap[i] * fabs(gradient_buf[i]);
+				result[i] += gmap[i] * std::fabs(gradient_buf[i]);
 				gmap[i] *= persistence_map[i];
 			}
 		} else {
 			for (size_t i = 0; i != bufsize; i++)
-				result[i] += g * fabs(gradient_buf[i]);
+				result[i] += g * std::fabs(gradient_buf[i]);
 		}
 	} else {
 		if (persistence_map) {
