@@ -21,8 +21,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 	Random portability stuff
 */
 
-#ifndef PORTING_HEADER
-#define PORTING_HEADER
+#pragma once
 
 #ifdef _WIN32
 	#ifdef _WIN32_WINNT
@@ -40,7 +39,6 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "debug.h"
 #include "constants.h"
 #include "gettime.h"
-#include "threads.h"
 
 #ifdef _MSC_VER
 	#define SWPRINTF_CHARSTRING L"%S"
@@ -58,11 +56,11 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 	#define sleep_ms(x) Sleep(x)
 #else
 	#include <unistd.h>
-	#include <stdint.h> //for uintptr_t
+	#include <cstdint> //for uintptr_t
 
 	// Use standard Posix macro for Linux
 	#if (defined(linux) || defined(__linux)) && !defined(__linux__)
-		#define __linux__ 
+		#define __linux__
 	#endif
 	#if (defined(__linux__) || defined(__GNU__)) && !defined(_GNU_SOURCE)
 		#define _GNU_SOURCE
@@ -114,8 +112,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #ifndef _WIN32 // Posix
 	#include <sys/time.h>
-	#include <time.h>
-	#if defined(__MACH__) && defined(__APPLE__)
+	#include <ctime>
+
+#if defined(__MACH__) && defined(__APPLE__)
 		#include <mach/clock.h>
 		#include <mach/mach.h>
 	#endif
@@ -128,10 +127,10 @@ namespace porting
 	Signal handler (grabs Ctrl-C on POSIX systems)
 */
 
-void signal_handler_init(void);
+void signal_handler_init();
 // Returns a pointer to a bool.
 // When the bool is true, program should quit.
-bool * signal_handler_killstatus(void);
+bool * signal_handler_killstatus();
 
 /*
 	Path of static data directory.
@@ -179,8 +178,6 @@ void initializePaths();
 */
 std::string get_sysinfo();
 
-void initIrrlicht(irr::IrrlichtDevice * );
-
 
 // Monotonic counter getters.
 
@@ -277,24 +274,11 @@ inline u64 getDeltaMs(u64 old_time_ms, u64 new_time_ms)
 {
 	if (new_time_ms >= old_time_ms) {
 		return (new_time_ms - old_time_ms);
-	} else {
-		return (old_time_ms - new_time_ms);
 	}
+
+	return (old_time_ms - new_time_ms);
 }
 
-
-#ifndef SERVER
-float getDisplayDensity();
-
-v2u32 getDisplaySize();
-v2u32 getWindowSize();
-
-std::vector<core::vector3d<u32> > getSupportedVideoModes();
-std::vector<irr::video::E_DRIVER_TYPE> getSupportedVideoDrivers();
-const char *getVideoDriverName(irr::video::E_DRIVER_TYPE type);
-const char *getVideoDriverFriendlyName(irr::video::E_DRIVER_TYPE type);
-#endif
-
 inline const char *getPlatformName()
 {
 	return
@@ -339,27 +323,14 @@ inline const char *getPlatformName()
 	;
 }
 
-void setXorgClassHint(const video::SExposedVideoData &video_data,
-	const std::string &name);
-
-bool setWindowIcon(IrrlichtDevice *device);
-
-bool setXorgWindowIconFromPath(IrrlichtDevice *device,
-	const std::string &icon_file);
-
-// This only needs to be called at the start of execution, since all future
-// threads in the process inherit this exception handler
-void setWin32ExceptionHandler();
-
 bool secure_rand_fill_buf(void *buf, size_t len);
 
 // This attaches to the parents process console, or creates a new one if it doesnt exist.
-void attachOrCreateConsole(void);
+void attachOrCreateConsole();
+
+int mt_snprintf(char *buf, const size_t buf_size, const char *fmt, ...);
 } // namespace porting
 
 #ifdef __ANDROID__
 #include "porting_android.h"
 #endif
-
-#endif // PORTING_HEADER
-
