@@ -26,6 +26,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "porting.h"  // strlcpy
 #include "server.h"
 #include "settings.h"
+#include "convert_json.h"
 
 /*
 	RemotePlayer
@@ -36,16 +37,7 @@ float RemotePlayer::m_setting_chat_message_limit_per_10sec = 0.0f;
 u16 RemotePlayer::m_setting_chat_message_limit_trigger_kick = 0;
 
 RemotePlayer::RemotePlayer(const char *name, IItemDefManager *idef):
-	Player(name, idef),
-	protocol_version(0),
-	m_sao(NULL),
-	m_dirty(false),
-	m_last_chat_message_sent(time(NULL)),
-	m_chat_message_allowance(5.0f),
-	m_message_rate_overhead(0),
-	m_day_night_ratio_do_override(false),
-	hud_hotbar_image(""),
-	hud_hotbar_selected_image("")
+	Player(name, idef)
 {
 	if (!RemotePlayer::m_setting_cache_loaded) {
 		RemotePlayer::m_setting_chat_message_limit_per_10sec =
@@ -80,14 +72,13 @@ void RemotePlayer::serializeExtraAttributes(std::string &output)
 {
 	assert(m_sao);
 	Json::Value json_root;
-	const PlayerAttributes &attrs = m_sao->getExtendedAttributes();
-	for (PlayerAttributes::const_iterator it = attrs.begin(); it != attrs.end(); ++it) {
-		json_root[(*it).first] = (*it).second;
+
+	const StringMap &attrs = m_sao->getMeta().getStrings();
+	for (const auto &attr : attrs) {
+		json_root[attr.first] = attr.second;
 	}
 
-	Json::FastWriter writer;
-	output = writer.write(json_root);
-	m_sao->setExtendedAttributeModified(false);
+	output = fastWriteJson(json_root);
 }
 
 
@@ -107,9 +98,9 @@ void RemotePlayer::deSerialize(std::istream &is, const std::string &playername,
 
 	if (sao) {
 		try {
-			sao->setHPRaw(args.getS32("hp"));
+			sao->setHPRaw(args.getU16("hp"));
 		} catch(SettingNotFoundException &e) {
-			sao->setHPRaw(PLAYER_MAX_HP);
+			sao->setHPRaw(PLAYER_MAX_HP_DEFAULT);
 		}
 
 		try {
@@ -117,34 +108,43 @@ void RemotePlayer::deSerialize(std::istream &is, const std::string &playername,
 		} catch (SettingNotFoundException &e) {}
 
 		try {
-			sao->setPitch(args.getFloat("pitch"));
+			sao->setLookPitch(args.getFloat("pitch"));
 		} catch (SettingNotFoundException &e) {}
 		try {
-			sao->setYaw(args.getFloat("yaw"));
+			sao->setPlayerYaw(args.getFloat("yaw"));
 		} catch (SettingNotFoundException &e) {}
 
 		try {
-			sao->setBreath(args.getS32("breath"), false);
+			sao->setBreath(args.getU16("breath"), false);
 		} catch (SettingNotFoundException &e) {}
 
 		try {
 			const std::string &extended_attributes = args.get("extended_attributes");
-			Json::Reader reader;
+			std::istringstream iss(extended_attributes);
+			Json::CharReaderBuilder builder;
+			builder.settings_["collectComments"] = false;
+			std::string errs;
+
 			Json::Value attr_root;
-			reader.parse(extended_attributes, attr_root);
+			Json::parseFromStream(builder, iss, &attr_root, &errs);
 
 			const Json::Value::Members attr_list = attr_root.getMemberNames();
-			for (Json::Value::Members::const_iterator it = attr_list.begin();
-					it != attr_list.end(); ++it) {
-				Json::Value attr_value = attr_root[*it];
-				sao->setExtendedAttribute(*it, attr_value.asString());
+			for (const auto &it : attr_list) {
+				Json::Value attr_value = attr_root[it];
+				sao->getMeta().setString(it, attr_value.asString());
 			}
+			sao->getMeta().setModified(false);
 		} catch (SettingNotFoundException &e) {}
 	}
 
-	inventory.deSerialize(is);
+	try {
+		inventory.deSerialize(is);
+	} catch (SerializationError &e) {
+		errorstream << "Failed to deserialize player inventory. player_name="
+			<< name << " " << e.what() << std::endl;
+	}
 
-	if (inventory.getList("craftpreview") == NULL) {
+	if (!inventory.getList("craftpreview") && inventory.getList("craftresult")) {
 		// Convert players without craftpreview
 		inventory.addList("craftpreview", 1);
 
@@ -168,13 +168,13 @@ void RemotePlayer::serialize(std::ostream &os)
 
 	// This should not happen
 	assert(m_sao);
-	args.setS32("hp", m_sao->getHP());
+	args.setU16("hp", m_sao->getHP());
 	args.setV3F("position", m_sao->getBasePosition());
-	args.setFloat("pitch", m_sao->getPitch());
-	args.setFloat("yaw", m_sao->getYaw());
-	args.setS32("breath", m_sao->getBreath());
+	args.setFloat("pitch", m_sao->getLookPitch());
+	args.setFloat("yaw", m_sao->getRotation().Y);
+	args.setU16("breath", m_sao->getBreath());
 
-	std::string extended_attrs = "";
+	std::string extended_attrs;
 	serializeExtraAttributes(extended_attrs);
 	args.set("extended_attributes", extended_attrs);
 
@@ -223,3 +223,10 @@ const RemotePlayerChatResult RemotePlayer::canSendChatMessage()
 	m_chat_message_allowance -= 1.0f;
 	return RPLAYER_CHATRESULT_OK;
 }
+
+void RemotePlayer::onSuccessfulSave()
+{
+	setModified(false);
+	if (m_sao)
+		m_sao->getMeta().setModified(false);
+}
