@@ -110,12 +110,7 @@ public:
 
 	bool checkModified() const { return m_dirty || inventory.checkModified(); }
 
-	void setModified(const bool x)
-	{
-		m_dirty = x;
-		if (!x)
-			inventory.setModified(x);
-	}
+	inline void setModified(const bool x) { m_dirty = x; }
 
 	void setLocalAnimations(v2s32 frames[4], float frame_speed)
 	{
@@ -135,6 +130,9 @@ public:
 
 	u16 protocol_version = 0;
 
+	// v1 for clients older than 5.1.0-dev
+	u16 formspec_version = 1;
+
 	session_t getPeerId() const { return m_peer_id; }
 
 	void setPeerId(session_t peer_id) { m_peer_id = peer_id; }
