@@ -21,6 +21,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "common/c_types.h"
 #include "nodedef.h"
 #include "object_properties.h"
+#include "content_sao.h"
 #include "cpp_api/s_node.h"
 #include "lua_api/l_object.h"
 #include "lua_api/l_item.h"
@@ -30,7 +31,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "tool.h"
 #include "serverobject.h"
 #include "porting.h"
-#include "mg_schematic.h"
+#include "mapgen/mg_schematic.h"
 #include "noise.h"
 #include "util/pointedthing.h"
 #include "debug.h" // For FATAL_ERROR
@@ -56,7 +57,9 @@ void read_item_definition(lua_State* L, int index,
 	getstringfield(L, index, "name", def.name);
 	getstringfield(L, index, "description", def.description);
 	getstringfield(L, index, "inventory_image", def.inventory_image);
+	getstringfield(L, index, "inventory_overlay", def.inventory_overlay);
 	getstringfield(L, index, "wield_image", def.wield_image);
+	getstringfield(L, index, "wield_overlay", def.wield_overlay);
 	getstringfield(L, index, "palette", def.palette_image);
 
 	// Read item color.
@@ -91,7 +94,7 @@ void read_item_definition(lua_State* L, int index,
 	// If name is "" (hand), ensure there are ToolCapabilities
 	// because it will be looked up there whenever any other item has
 	// no ToolCapabilities
-	if(def.name == "" && def.tool_capabilities == NULL){
+	if (def.name.empty() && def.tool_capabilities == NULL){
 		def.tool_capabilities = new ToolCapabilities();
 	}
 
@@ -132,7 +135,7 @@ void push_item_definition(lua_State *L, const ItemDefinition &i)
 void push_item_definition_full(lua_State *L, const ItemDefinition &i)
 {
 	std::string type(es_ItemType[(int)i.type].str);
-	
+
 	lua_newtable(L);
 	lua_pushstring(L, i.name.c_str());
 	lua_setfield(L, -2, "name");
@@ -142,8 +145,12 @@ void push_item_definition_full(lua_State *L, const ItemDefinition &i)
 	lua_setfield(L, -2, "type");
 	lua_pushstring(L, i.inventory_image.c_str());
 	lua_setfield(L, -2, "inventory_image");
+	lua_pushstring(L, i.inventory_overlay.c_str());
+	lua_setfield(L, -2, "inventory_overlay");
 	lua_pushstring(L, i.wield_image.c_str());
 	lua_setfield(L, -2, "wield_image");
+	lua_pushstring(L, i.wield_overlay.c_str());
+	lua_setfield(L, -2, "wield_overlay");
 	lua_pushstring(L, i.palette_image.c_str());
 	lua_setfield(L, -2, "palette_image");
 	push_ARGB8(L, i.color);
@@ -157,11 +164,7 @@ void push_item_definition_full(lua_State *L, const ItemDefinition &i)
 	lua_pushboolean(L, i.liquids_pointable);
 	lua_setfield(L, -2, "liquids_pointable");
 	if (i.type == ITEM_TOOL) {
-		push_tool_capabilities(L, ToolCapabilities(
-			i.tool_capabilities->full_punch_interval,
-			i.tool_capabilities->max_drop_level,
-			i.tool_capabilities->groupcaps,
-			i.tool_capabilities->damageGroups));
+		push_tool_capabilities(L, *i.tool_capabilities);
 		lua_setfield(L, -2, "tool_capabilities");
 	}
 	push_groups(L, i.groups);
@@ -176,32 +179,68 @@ void push_item_definition_full(lua_State *L, const ItemDefinition &i)
 
 /******************************************************************************/
 void read_object_properties(lua_State *L, int index,
-		ObjectProperties *prop, IItemDefManager *idef)
+		ServerActiveObject *sao, ObjectProperties *prop, IItemDefManager *idef)
 {
 	if(index < 0)
 		index = lua_gettop(L) + 1 + index;
 	if(!lua_istable(L, index))
 		return;
 
-	prop->hp_max = getintfield_default(L, -1, "hp_max", 10);
+	int hp_max = 0;
+	if (getintfield(L, -1, "hp_max", hp_max)) {
+		prop->hp_max = (u16)rangelim(hp_max, 0, U16_MAX);
 
+		if (prop->hp_max < sao->getHP()) {
+			PlayerHPChangeReason reason(PlayerHPChangeReason::SET_HP);
+			sao->setHP(prop->hp_max, reason);
+			if (sao->getType() == ACTIVEOBJECT_TYPE_PLAYER)
+				sao->getEnv()->getGameDef()->SendPlayerHPOrDie((PlayerSAO *)sao, reason);
+		}
+	}
+
+	if (getintfield(L, -1, "breath_max", prop->breath_max)) {
+		if (sao->getType() == ACTIVEOBJECT_TYPE_PLAYER) {
+			PlayerSAO *player = (PlayerSAO *)sao;
+			if (prop->breath_max < player->getBreath())
+				player->setBreath(prop->breath_max);
+		}
+	}
 	getboolfield(L, -1, "physical", prop->physical);
 	getboolfield(L, -1, "collide_with_objects", prop->collideWithObjects);
 
 	getfloatfield(L, -1, "weight", prop->weight);
 
 	lua_getfield(L, -1, "collisionbox");
-	if(lua_istable(L, -1))
+	bool collisionbox_defined = lua_istable(L, -1);
+	if (collisionbox_defined)
 		prop->collisionbox = read_aabb3f(L, -1, 1.0);
 	lua_pop(L, 1);
 
+	lua_getfield(L, -1, "selectionbox");
+	if (lua_istable(L, -1))
+		prop->selectionbox = read_aabb3f(L, -1, 1.0);
+	else if (collisionbox_defined)
+		prop->selectionbox = prop->collisionbox;
+	lua_pop(L, 1);
+
+	getboolfield(L, -1, "pointable", prop->pointable);
 	getstringfield(L, -1, "visual", prop->visual);
 
 	getstringfield(L, -1, "mesh", prop->mesh);
 
 	lua_getfield(L, -1, "visual_size");
-	if(lua_istable(L, -1))
-		prop->visual_size = read_v2f(L, -1);
+	if (lua_istable(L, -1)) {
+		// Backwards compatibility: Also accept { x = ?, y = ? }
+		v2f scale_xy = read_v2f(L, -1);
+
+		f32 scale_z = scale_xy.X;
+		lua_getfield(L, -1, "z");
+		if (lua_isnumber(L, -1))
+			scale_z = lua_tonumber(L, -1);
+		lua_pop(L, 1);
+
+		prop->visual_size = v3f(scale_xy.X, scale_xy.Y, scale_z);
+	}
 	lua_pop(L, 1);
 
 	lua_getfield(L, -1, "textures");
@@ -212,9 +251,9 @@ void read_object_properties(lua_State *L, int index,
 		while(lua_next(L, table) != 0){
 			// key at index -2 and value at index -1
 			if(lua_isstring(L, -1))
-				prop->textures.push_back(lua_tostring(L, -1));
+				prop->textures.emplace_back(lua_tostring(L, -1));
 			else
-				prop->textures.push_back("");
+				prop->textures.emplace_back("");
 			// removes value, keeps key for next iteration
 			lua_pop(L, 1);
 		}
@@ -245,9 +284,11 @@ void read_object_properties(lua_State *L, int index,
 
 	getboolfield(L, -1, "is_visible", prop->is_visible);
 	getboolfield(L, -1, "makes_footstep_sound", prop->makes_footstep_sound);
-	getfloatfield(L, -1, "automatic_rotate", prop->automatic_rotate);
 	if (getfloatfield(L, -1, "stepheight", prop->stepheight))
 		prop->stepheight *= BS;
+	getfloatfield(L, -1, "eye_height", prop->eye_height);
+
+	getfloatfield(L, -1, "automatic_rotate", prop->automatic_rotate);
 	lua_getfield(L, -1, "automatic_face_movement_dir");
 	if (lua_isnumber(L, -1)) {
 		prop->automatic_face_movement_dir = true;
@@ -258,6 +299,7 @@ void read_object_properties(lua_State *L, int index,
 	}
 	lua_pop(L, 1);
 	getboolfield(L, -1, "backface_culling", prop->backface_culling);
+	getintfield(L, -1, "glow", prop->glow);
 
 	getstringfield(L, -1, "nametag", prop->nametag);
 	lua_getfield(L, -1, "nametag_color");
@@ -273,11 +315,17 @@ void read_object_properties(lua_State *L, int index,
 		prop->automatic_face_movement_max_rotation_per_sec = luaL_checknumber(L, -1);
 	}
 	lua_pop(L, 1);
+
 	getstringfield(L, -1, "infotext", prop->infotext);
+	getboolfield(L, -1, "static_save", prop->static_save);
+
 	lua_getfield(L, -1, "wield_item");
 	if (!lua_isnil(L, -1))
 		prop->wield_item = read_item(L, -1, idef).getItemString();
 	lua_pop(L, 1);
+
+	getfloatfield(L, -1, "zoom_fov", prop->zoom_fov);
+	getboolfield(L, -1, "use_texture_alpha", prop->use_texture_alpha);
 }
 
 /******************************************************************************/
@@ -286,6 +334,8 @@ void push_object_properties(lua_State *L, ObjectProperties *prop)
 	lua_newtable(L);
 	lua_pushnumber(L, prop->hp_max);
 	lua_setfield(L, -2, "hp_max");
+	lua_pushnumber(L, prop->breath_max);
+	lua_setfield(L, -2, "breath_max");
 	lua_pushboolean(L, prop->physical);
 	lua_setfield(L, -2, "physical");
 	lua_pushboolean(L, prop->collideWithObjects);
@@ -294,28 +344,30 @@ void push_object_properties(lua_State *L, ObjectProperties *prop)
 	lua_setfield(L, -2, "weight");
 	push_aabb3f(L, prop->collisionbox);
 	lua_setfield(L, -2, "collisionbox");
+	push_aabb3f(L, prop->selectionbox);
+	lua_setfield(L, -2, "selectionbox");
+	lua_pushboolean(L, prop->pointable);
+	lua_setfield(L, -2, "pointable");
 	lua_pushlstring(L, prop->visual.c_str(), prop->visual.size());
 	lua_setfield(L, -2, "visual");
 	lua_pushlstring(L, prop->mesh.c_str(), prop->mesh.size());
 	lua_setfield(L, -2, "mesh");
-	push_v2f(L, prop->visual_size);
+	push_v3f(L, prop->visual_size);
 	lua_setfield(L, -2, "visual_size");
 
 	lua_newtable(L);
 	u16 i = 1;
-	for (std::vector<std::string>::iterator it = prop->textures.begin();
-			it != prop->textures.end(); ++it) {
-		lua_pushlstring(L, it->c_str(), it->size());
-		lua_rawseti(L, -2, i);
+	for (const std::string &texture : prop->textures) {
+		lua_pushlstring(L, texture.c_str(), texture.size());
+		lua_rawseti(L, -2, i++);
 	}
 	lua_setfield(L, -2, "textures");
 
 	lua_newtable(L);
 	i = 1;
-	for (std::vector<video::SColor>::iterator it = prop->colors.begin();
-			it != prop->colors.end(); ++it) {
-		push_ARGB8(L, *it);
-		lua_rawseti(L, -2, i);
+	for (const video::SColor &color : prop->colors) {
+		push_ARGB8(L, color);
+		lua_rawseti(L, -2, i++);
 	}
 	lua_setfield(L, -2, "colors");
 
@@ -327,10 +379,12 @@ void push_object_properties(lua_State *L, ObjectProperties *prop)
 	lua_setfield(L, -2, "is_visible");
 	lua_pushboolean(L, prop->makes_footstep_sound);
 	lua_setfield(L, -2, "makes_footstep_sound");
-	lua_pushnumber(L, prop->automatic_rotate);
-	lua_setfield(L, -2, "automatic_rotate");
 	lua_pushnumber(L, prop->stepheight / BS);
 	lua_setfield(L, -2, "stepheight");
+	lua_pushnumber(L, prop->eye_height);
+	lua_setfield(L, -2, "eye_height");
+	lua_pushnumber(L, prop->automatic_rotate);
+	lua_setfield(L, -2, "automatic_rotate");
 	if (prop->automatic_face_movement_dir)
 		lua_pushnumber(L, prop->automatic_face_movement_dir_offset);
 	else
@@ -338,6 +392,8 @@ void push_object_properties(lua_State *L, ObjectProperties *prop)
 	lua_setfield(L, -2, "automatic_face_movement_dir");
 	lua_pushboolean(L, prop->backface_culling);
 	lua_setfield(L, -2, "backface_culling");
+	lua_pushnumber(L, prop->glow);
+	lua_setfield(L, -2, "glow");
 	lua_pushlstring(L, prop->nametag.c_str(), prop->nametag.size());
 	lua_setfield(L, -2, "nametag");
 	push_ARGB8(L, prop->nametag_color);
@@ -346,8 +402,14 @@ void push_object_properties(lua_State *L, ObjectProperties *prop)
 	lua_setfield(L, -2, "automatic_face_movement_max_rotation_per_sec");
 	lua_pushlstring(L, prop->infotext.c_str(), prop->infotext.size());
 	lua_setfield(L, -2, "infotext");
+	lua_pushboolean(L, prop->static_save);
+	lua_setfield(L, -2, "static_save");
 	lua_pushlstring(L, prop->wield_item.c_str(), prop->wield_item.size());
 	lua_setfield(L, -2, "wield_item");
+	lua_pushnumber(L, prop->zoom_fov);
+	lua_setfield(L, -2, "zoom_fov");
+	lua_pushboolean(L, prop->use_texture_alpha);
+	lua_setfield(L, -2, "use_texture_alpha");
 }
 
 /******************************************************************************/
@@ -362,6 +424,7 @@ TileDef read_tiledef(lua_State *L, int index, u8 drawtype)
 	bool default_culling = true;
 	switch (drawtype) {
 		case NDT_PLANTLIKE:
+		case NDT_PLANTLIKE_ROOTED:
 		case NDT_FIRELIKE:
 			default_tiling = false;
 			// "break" is omitted here intentionaly, as PLANTLIKE
@@ -395,6 +458,16 @@ TileDef read_tiledef(lua_State *L, int index, u8 drawtype)
 			L, index, "tileable_horizontal", default_tiling);
 		tiledef.tileable_vertical = getboolfield_default(
 			L, index, "tileable_vertical", default_tiling);
+		std::string align_style;
+		if (getstringfield(L, index, "align_style", align_style)) {
+			if (align_style == "user")
+				tiledef.align_style = ALIGN_STYLE_USER_DEFINED;
+			else if (align_style == "world")
+				tiledef.align_style = ALIGN_STYLE_WORLD;
+			else
+				tiledef.align_style = ALIGN_STYLE_NODE;
+		}
+		tiledef.scale = getintfield_default(L, index, "scale", 0);
 		// color = ...
 		lua_getfield(L, index, "color");
 		tiledef.has_color = read_color(L, -1, &tiledef.color);
@@ -563,7 +636,7 @@ ContentFeatures read_content_features(lua_State *L, int index)
 	f.param_type_2 = (ContentParamType2)getenumfield(L, index, "paramtype2",
 			ScriptApiNode::es_ContentParamType2, CPT2_NONE);
 
-	if (f.palette_name != "" &&
+	if (!f.palette_name.empty() &&
 			!(f.param_type_2 == CPT2_COLOR ||
 			f.param_type_2 == CPT2_COLORED_FACEDIR ||
 			f.param_type_2 == CPT2_COLORED_WALLMOUNTED))
@@ -645,7 +718,7 @@ ContentFeatures read_content_features(lua_State *L, int index)
 		lua_pushnil(L);
 		while (lua_next(L, table) != 0) {
 			// Value at -1
-			f.connects_to.push_back(lua_tostring(L, -1));
+			f.connects_to.emplace_back(lua_tostring(L, -1));
 			lua_pop(L, 1);
 		}
 	}
@@ -712,6 +785,10 @@ ContentFeatures read_content_features(lua_State *L, int index)
 	}
 	lua_pop(L, 1);
 
+	// Node immediately placed by client when node is dug
+	getstringfield(L, index, "node_dig_prediction",
+		f.node_dig_prediction);
+
 	return f;
 }
 
@@ -721,9 +798,9 @@ void push_content_features(lua_State *L, const ContentFeatures &c)
 	std::string paramtype2(ScriptApiNode::es_ContentParamType2[(int)c.param_type_2].str);
 	std::string drawtype(ScriptApiNode::es_DrawType[(int)c.drawtype].str);
 	std::string liquid_type(ScriptApiNode::es_LiquidType[(int)c.liquid_type].str);
-	
+
 	/* Missing "tiles" because I don't see a usecase (at least not yet). */
-	
+
 	lua_newtable(L);
 	lua_pushboolean(L, c.has_on_construct);
 	lua_setfield(L, -2, "has_on_construct");
@@ -756,10 +833,10 @@ void push_content_features(lua_State *L, const ContentFeatures &c)
 	if (!c.palette_name.empty()) {
 		push_ARGB8(L, c.color);
 		lua_setfield(L, -2, "color");
-		
+
 		lua_pushstring(L, c.palette_name.c_str());
 		lua_setfield(L, -2, "palette_name");
-		
+
 		push_palette(L, c.palette);
 		lua_setfield(L, -2, "palette");
 	}
@@ -767,16 +844,15 @@ void push_content_features(lua_State *L, const ContentFeatures &c)
 	lua_setfield(L, -2, "waving");
 	lua_pushnumber(L, c.connect_sides);
 	lua_setfield(L, -2, "connect_sides");
-	
+
 	lua_newtable(L);
 	u16 i = 1;
-	for (std::vector<std::string>::const_iterator it = c.connects_to.begin();
-			it != c.connects_to.end(); ++it) {
-		lua_pushlstring(L, it->c_str(), it->size());
-		lua_rawseti(L, -2, i);
+	for (const std::string &it : c.connects_to) {
+		lua_pushlstring(L, it.c_str(), it.size());
+		lua_rawseti(L, -2, i++);
 	}
 	lua_setfield(L, -2, "connects_to");
-	
+
 	push_ARGB8(L, c.post_effect_color);
 	lua_setfield(L, -2, "post_effect_color");
 	lua_pushnumber(L, c.leveled);
@@ -837,6 +913,8 @@ void push_content_features(lua_State *L, const ContentFeatures &c)
 	lua_setfield(L, -2, "legacy_facedir_simple");
 	lua_pushboolean(L, c.legacy_wallmounted);
 	lua_setfield(L, -2, "legacy_wallmounted");
+	lua_pushstring(L, c.node_dig_prediction.c_str());
+	lua_setfield(L, -2, "node_dig_prediction");
 }
 
 /******************************************************************************/
@@ -892,10 +970,9 @@ void push_box(lua_State *L, const std::vector<aabb3f> &box)
 {
 	lua_newtable(L);
 	u8 i = 1;
-	for (std::vector<aabb3f>::const_iterator it = box.begin();
-	                it != box.end(); ++it) {
-		push_aabb3f(L, (*it));
-		lua_rawseti(L, -2, i);
+	for (const aabb3f &it : box) {
+		push_aabb3f(L, it);
+		lua_rawseti(L, -2, i++);
 	}
 }
 
@@ -925,6 +1002,7 @@ void read_server_sound_params(lua_State *L, int index,
 		getfloatfield(L, index, "gain", params.gain);
 		getstringfield(L, index, "to_player", params.to_player);
 		getfloatfield(L, index, "fade", params.fade);
+		getfloatfield(L, index, "pitch", params.pitch);
 		lua_getfield(L, index, "pos");
 		if(!lua_isnil(L, -1)){
 			v3f p = read_v3f(L, -1)*BS;
@@ -958,6 +1036,7 @@ void read_soundspec(lua_State *L, int index, SimpleSoundSpec &spec)
 		getstringfield(L, index, "name", spec.name);
 		getfloatfield(L, index, "gain", spec.gain);
 		getfloatfield(L, index, "fade", spec.fade);
+		getfloatfield(L, index, "pitch", spec.pitch);
 	} else if(lua_isstring(L, index)){
 		spec.name = lua_tostring(L, index);
 	}
@@ -972,6 +1051,8 @@ void push_soundspec(lua_State *L, const SimpleSoundSpec &spec)
 	lua_setfield(L, -2, "gain");
 	lua_pushnumber(L, spec.fade);
 	lua_setfield(L, -2, "fade");
+	lua_pushnumber(L, spec.pitch);
+	lua_setfield(L, -2, "pitch");
 }
 
 /******************************************************************************/
@@ -982,21 +1063,19 @@ NodeBox read_nodebox(lua_State *L, int index)
 		nodebox.type = (NodeBoxType)getenumfield(L, index, "type",
 				ScriptApiNode::es_NodeBoxType, NODEBOX_REGULAR);
 
-#define NODEBOXREAD(n, s) \
-	do { \
+#define NODEBOXREAD(n, s){ \
 		lua_getfield(L, index, (s)); \
 		if (lua_istable(L, -1)) \
 			(n) = read_aabb3f(L, -1, BS); \
 		lua_pop(L, 1); \
-	} while (0)
+	}
 
 #define NODEBOXREADVEC(n, s) \
-	do { \
 		lua_getfield(L, index, (s)); \
 		if (lua_istable(L, -1)) \
 			(n) = read_aabb3f_vector(L, -1, BS); \
-		lua_pop(L, 1); \
-	} while (0)
+		lua_pop(L, 1);
+
 		NODEBOXREADVEC(nodebox.fixed, "fixed");
 		NODEBOXREAD(nodebox.wall_top, "wall_top");
 		NODEBOXREAD(nodebox.wall_bottom, "wall_bottom");
@@ -1007,17 +1086,25 @@ NodeBox read_nodebox(lua_State *L, int index)
 		NODEBOXREADVEC(nodebox.connect_left, "connect_left");
 		NODEBOXREADVEC(nodebox.connect_back, "connect_back");
 		NODEBOXREADVEC(nodebox.connect_right, "connect_right");
+		NODEBOXREADVEC(nodebox.disconnected_top, "disconnected_top");
+		NODEBOXREADVEC(nodebox.disconnected_bottom, "disconnected_bottom");
+		NODEBOXREADVEC(nodebox.disconnected_front, "disconnected_front");
+		NODEBOXREADVEC(nodebox.disconnected_left, "disconnected_left");
+		NODEBOXREADVEC(nodebox.disconnected_back, "disconnected_back");
+		NODEBOXREADVEC(nodebox.disconnected_right, "disconnected_right");
+		NODEBOXREADVEC(nodebox.disconnected, "disconnected");
+		NODEBOXREADVEC(nodebox.disconnected_sides, "disconnected_sides");
 	}
 	return nodebox;
 }
 
 /******************************************************************************/
-MapNode readnode(lua_State *L, int index, INodeDefManager *ndef)
+MapNode readnode(lua_State *L, int index, const NodeDefManager *ndef)
 {
 	lua_getfield(L, index, "name");
 	if (!lua_isstring(L, -1))
 		throw LuaError("Node name is not set or is not a string!");
-	const char *name = lua_tostring(L, -1);
+	std::string name = lua_tostring(L, -1);
 	lua_pop(L, 1);
 
 	u8 param1 = 0;
@@ -1032,11 +1119,15 @@ MapNode readnode(lua_State *L, int index, INodeDefManager *ndef)
 		param2 = lua_tonumber(L, -1);
 	lua_pop(L, 1);
 
-	return MapNode(ndef, name, param1, param2);
+	content_t id = CONTENT_IGNORE;
+	if (!ndef->getId(name, id))
+		throw LuaError("\"" + name + "\" is not a registered node!");
+
+	return {id, param1, param2};
 }
 
 /******************************************************************************/
-void pushnode(lua_State *L, const MapNode &n, INodeDefManager *ndef)
+void pushnode(lua_State *L, const MapNode &n, const NodeDefManager *ndef)
 {
 	lua_newtable(L);
 	lua_pushstring(L, ndef->get(n).name.c_str());
@@ -1091,18 +1182,17 @@ ItemStack read_item(lua_State* L, int index, IItemDefManager *idef)
 	if(index < 0)
 		index = lua_gettop(L) + 1 + index;
 
-	if(lua_isnil(L, index))
-	{
+	if (lua_isnil(L, index)) {
 		return ItemStack();
 	}
-	else if(lua_isuserdata(L, index))
-	{
+
+	if (lua_isuserdata(L, index)) {
 		// Convert from LuaItemStack
 		LuaItemStack *o = LuaItemStack::checkobject(L, index);
 		return o->getItem();
 	}
-	else if(lua_isstring(L, index))
-	{
+
+	if (lua_isstring(L, index)) {
 		// Convert from itemstring
 		std::string itemstring = lua_tostring(L, index);
 		try
@@ -1159,22 +1249,21 @@ void push_tool_capabilities(lua_State *L,
 {
 	lua_newtable(L);
 	setfloatfield(L, -1, "full_punch_interval", toolcap.full_punch_interval);
-		setintfield(L, -1, "max_drop_level", toolcap.max_drop_level);
+	setintfield(L, -1, "max_drop_level", toolcap.max_drop_level);
+	setintfield(L, -1, "punch_attack_uses", toolcap.punch_attack_uses);
 		// Create groupcaps table
 		lua_newtable(L);
 		// For each groupcap
-		for (ToolGCMap::const_iterator i = toolcap.groupcaps.begin();
-			i != toolcap.groupcaps.end(); ++i) {
+		for (const auto &gc_it : toolcap.groupcaps) {
 			// Create groupcap table
 			lua_newtable(L);
-			const std::string &name = i->first;
-			const ToolGroupCap &groupcap = i->second;
+			const std::string &name = gc_it.first;
+			const ToolGroupCap &groupcap = gc_it.second;
 			// Create subtable "times"
 			lua_newtable(L);
-			for (UNORDERED_MAP<int, float>::const_iterator
-					i = groupcap.times.begin(); i != groupcap.times.end(); ++i) {
-				lua_pushinteger(L, i->first);
-				lua_pushnumber(L, i->second);
+			for (auto time : groupcap.times) {
+				lua_pushinteger(L, time.first);
+				lua_pushnumber(L, time.second);
 				lua_settable(L, -3);
 			}
 			// Set subtable "times"
@@ -1190,11 +1279,10 @@ void push_tool_capabilities(lua_State *L,
 		//Create damage_groups table
 		lua_newtable(L);
 		// For each damage group
-		for (DamageGroup::const_iterator i = toolcap.damageGroups.begin();
-			i != toolcap.damageGroups.end(); ++i) {
+		for (const auto &damageGroup : toolcap.damageGroups) {
 			// Create damage group table
-			lua_pushinteger(L, i->second);
-			lua_setfield(L, -2, i->first.c_str());
+			lua_pushinteger(L, damageGroup.second);
+			lua_setfield(L, -2, damageGroup.first.c_str());
 		}
 		lua_setfield(L, -2, "damage_groups");
 }
@@ -1284,6 +1372,7 @@ ToolCapabilities read_tool_capabilities(
 	ToolCapabilities toolcap;
 	getfloatfield(L, table, "full_punch_interval", toolcap.full_punch_interval);
 	getintfield(L, table, "max_drop_level", toolcap.max_drop_level);
+	getintfield(L, table, "punch_attack_uses", toolcap.punch_attack_uses);
 	lua_getfield(L, table, "groupcaps");
 	if(lua_istable(L, -1)){
 		int table_groupcaps = lua_gettop(L);
@@ -1438,13 +1527,15 @@ void read_groups(lua_State *L, int index, ItemGroupList &result)
 		return;
 	result.clear();
 	lua_pushnil(L);
-	if(index < 0)
+	if (index < 0)
 		index -= 1;
-	while(lua_next(L, index) != 0){
+	while (lua_next(L, index) != 0) {
 		// key at index -2 and value at index -1
 		std::string name = luaL_checkstring(L, -2);
 		int rating = luaL_checkinteger(L, -1);
-		result[name] = rating;
+		// zero rating indicates not in the group
+		if (rating != 0)
+			result[name] = rating;
 		// removes value, keeps key for next iteration
 		lua_pop(L, 1);
 	}
@@ -1454,9 +1545,9 @@ void read_groups(lua_State *L, int index, ItemGroupList &result)
 void push_groups(lua_State *L, const ItemGroupList &groups)
 {
 	lua_newtable(L);
-	for (ItemGroupList::const_iterator it = groups.begin(); it != groups.end(); ++it) {
-		lua_pushnumber(L, it->second);
-		lua_setfield(L, -2, it->first.c_str());
+	for (const auto &group : groups) {
+		lua_pushnumber(L, group.second);
+		lua_setfield(L, -2, group.first.c_str());
 	}
 }
 
@@ -1538,13 +1629,13 @@ bool read_noiseparams(lua_State *L, int index, NoiseParams *np)
 void push_noiseparams(lua_State *L, NoiseParams *np)
 {
 	lua_newtable(L);
-	lua_pushnumber(L, np->offset);
+	push_float_string(L, np->offset);
 	lua_setfield(L, -2, "offset");
-	lua_pushnumber(L, np->scale);
+	push_float_string(L, np->scale);
 	lua_setfield(L, -2, "scale");
-	lua_pushnumber(L, np->persist);
+	push_float_string(L, np->persist);
 	lua_setfield(L, -2, "persistence");
-	lua_pushnumber(L, np->lacunarity);
+	push_float_string(L, np->lacunarity);
 	lua_setfield(L, -2, "lacunarity");
 	lua_pushnumber(L, np->seed);
 	lua_setfield(L, -2, "seed");
@@ -1555,7 +1646,7 @@ void push_noiseparams(lua_State *L, NoiseParams *np)
 		np->flags);
 	lua_setfield(L, -2, "flags");
 
-	push_v3f(L, np->spread);
+	push_v3_float_string(L, np->spread);
 	lua_setfield(L, -2, "spread");
 }
 
@@ -1567,9 +1658,8 @@ static int push_json_value_getdepth(const Json::Value &value)
 		return 1;
 
 	int maxdepth = 0;
-	for (Json::Value::const_iterator it = value.begin();
-			it != value.end(); ++it) {
-		int elemdepth = push_json_value_getdepth(*it);
+	for (const auto &it : value) {
+		int elemdepth = push_json_value_getdepth(it);
 		if (elemdepth > maxdepth)
 			maxdepth = elemdepth;
 	}
@@ -1641,8 +1731,8 @@ bool push_json_value(lua_State *L, const Json::Value &value, int nullindex)
 	// of push_json_value_helper is 2, so make sure there a depth * 2 slots
 	if (lua_checkstack(L, depth * 2))
 		return push_json_value_helper(L, value, nullindex);
-	else
-		return false;
+
+	return false;
 }
 
 // Converts Lua table --> JSON
@@ -1696,7 +1786,8 @@ void read_json_value(lua_State *L, Json::Value &root, int index, u8 recursion)
 	lua_pop(L, 1); // Pop value
 }
 
-void push_pointed_thing(lua_State *L, const PointedThing &pointed, bool csm)
+void push_pointed_thing(lua_State *L, const PointedThing &pointed, bool csm,
+	bool hitpoint)
 {
 	lua_newtable(L);
 	if (pointed.type == POINTEDTHING_NODE) {
@@ -1721,6 +1812,14 @@ void push_pointed_thing(lua_State *L, const PointedThing &pointed, bool csm)
 		lua_pushstring(L, "nothing");
 		lua_setfield(L, -2, "type");
 	}
+	if (hitpoint && (pointed.type != POINTEDTHING_NOTHING)) {
+		push_v3f(L, pointed.intersection_point / BS); // convert to node coords
+		lua_setfield(L, -2, "intersection_point");
+		push_v3s16(L, pointed.intersection_normal);
+		lua_setfield(L, -2, "intersection_normal");
+		lua_pushinteger(L, pointed.box_id + 1); // change to Lua array index
+		lua_setfield(L, -2, "box_id");
+	}
 }
 
 void push_objectRef(lua_State *L, const u16 id)
@@ -1734,3 +1833,151 @@ void push_objectRef(lua_State *L, const u16 id)
 	lua_remove(L, -2); // object_refs
 	lua_remove(L, -2); // core
 }
+
+void read_hud_element(lua_State *L, HudElement *elem)
+{
+	elem->type = (HudElementType)getenumfield(L, 2, "hud_elem_type",
+									es_HudElementType, HUD_ELEM_TEXT);
+
+	lua_getfield(L, 2, "position");
+	elem->pos = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
+	lua_pop(L, 1);
+
+	lua_getfield(L, 2, "scale");
+	elem->scale = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
+	lua_pop(L, 1);
+
+	lua_getfield(L, 2, "size");
+	elem->size = lua_istable(L, -1) ? read_v2s32(L, -1) : v2s32();
+	lua_pop(L, 1);
+
+	elem->name   = getstringfield_default(L, 2, "name", "");
+	elem->text   = getstringfield_default(L, 2, "text", "");
+	elem->number = getintfield_default(L, 2, "number", 0);
+	elem->item   = getintfield_default(L, 2, "item", 0);
+	elem->dir    = getintfield_default(L, 2, "direction", 0);
+
+	// Deprecated, only for compatibility's sake
+	if (elem->dir == 0)
+		elem->dir = getintfield_default(L, 2, "dir", 0);
+
+	lua_getfield(L, 2, "alignment");
+	elem->align = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
+	lua_pop(L, 1);
+
+	lua_getfield(L, 2, "offset");
+	elem->offset = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
+	lua_pop(L, 1);
+
+	lua_getfield(L, 2, "world_pos");
+	elem->world_pos = lua_istable(L, -1) ? read_v3f(L, -1) : v3f();
+	lua_pop(L, 1);
+
+	/* check for known deprecated element usage */
+	if ((elem->type  == HUD_ELEM_STATBAR) && (elem->size == v2s32()))
+		log_deprecated(L,"Deprecated usage of statbar without size!");
+}
+
+void push_hud_element(lua_State *L, HudElement *elem)
+{
+	lua_newtable(L);
+
+	lua_pushstring(L, es_HudElementType[(u8)elem->type].str);
+	lua_setfield(L, -2, "type");
+
+	push_v2f(L, elem->pos);
+	lua_setfield(L, -2, "position");
+
+	lua_pushstring(L, elem->name.c_str());
+	lua_setfield(L, -2, "name");
+
+	push_v2f(L, elem->scale);
+	lua_setfield(L, -2, "scale");
+
+	lua_pushstring(L, elem->text.c_str());
+	lua_setfield(L, -2, "text");
+
+	lua_pushnumber(L, elem->number);
+	lua_setfield(L, -2, "number");
+
+	lua_pushnumber(L, elem->item);
+	lua_setfield(L, -2, "item");
+
+	lua_pushnumber(L, elem->dir);
+	lua_setfield(L, -2, "direction");
+
+	push_v2f(L, elem->offset);
+	lua_setfield(L, -2, "offset");
+
+	push_v2f(L, elem->align);
+	lua_setfield(L, -2, "alignment");
+
+	push_v2s32(L, elem->size);
+	lua_setfield(L, -2, "size");
+
+	// Deprecated, only for compatibility's sake
+	lua_pushnumber(L, elem->dir);
+	lua_setfield(L, -2, "dir");
+
+	push_v3f(L, elem->world_pos);
+	lua_setfield(L, -2, "world_pos");
+}
+
+HudElementStat read_hud_change(lua_State *L, HudElement *elem, void **value)
+{
+	HudElementStat stat = HUD_STAT_NUMBER;
+	if (lua_isstring(L, 3)) {
+		int statint;
+		std::string statstr = lua_tostring(L, 3);
+		stat = string_to_enum(es_HudElementStat, statint, statstr) ?
+				(HudElementStat)statint : stat;
+	}
+
+	switch (stat) {
+		case HUD_STAT_POS:
+			elem->pos = read_v2f(L, 4);
+			*value = &elem->pos;
+			break;
+		case HUD_STAT_NAME:
+			elem->name = luaL_checkstring(L, 4);
+			*value = &elem->name;
+			break;
+		case HUD_STAT_SCALE:
+			elem->scale = read_v2f(L, 4);
+			*value = &elem->scale;
+			break;
+		case HUD_STAT_TEXT:
+			elem->text = luaL_checkstring(L, 4);
+			*value = &elem->text;
+			break;
+		case HUD_STAT_NUMBER:
+			elem->number = luaL_checknumber(L, 4);
+			*value = &elem->number;
+			break;
+		case HUD_STAT_ITEM:
+			elem->item = luaL_checknumber(L, 4);
+			*value = &elem->item;
+			break;
+		case HUD_STAT_DIR:
+			elem->dir = luaL_checknumber(L, 4);
+			*value = &elem->dir;
+			break;
+		case HUD_STAT_ALIGN:
+			elem->align = read_v2f(L, 4);
+			*value = &elem->align;
+			break;
+		case HUD_STAT_OFFSET:
+			elem->offset = read_v2f(L, 4);
+			*value = &elem->offset;
+			break;
+		case HUD_STAT_WORLD_POS:
+			elem->world_pos = read_v3f(L, 4);
+			*value = &elem->world_pos;
+			break;
+		case HUD_STAT_SIZE:
+			elem->size = read_v2s32(L, 4);
+			*value = &elem->size;
+			break;
+	}
+	return stat;
+}
