@@ -20,41 +20,16 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "common/c_internal.h"
 #include "debug.h"
 #include "log.h"
+#include "porting.h"
 #include "settings.h"
 
 std::string script_get_backtrace(lua_State *L)
 {
-	std::string s;
-	lua_getglobal(L, "debug");
-	if(lua_istable(L, -1)){
-		lua_getfield(L, -1, "traceback");
-		if(lua_isfunction(L, -1)) {
-			lua_call(L, 0, 1);
-			if(lua_isstring(L, -1)){
-				s = lua_tostring(L, -1);
-			}
-		}
-		lua_pop(L, 1);
-	}
+	lua_rawgeti(L, LUA_REGISTRYINDEX, CUSTOM_RIDX_BACKTRACE);
+	lua_call(L, 0, 1);
+	std::string result = luaL_checkstring(L, -1);
 	lua_pop(L, 1);
-	return s;
-}
-
-int script_error_handler(lua_State *L) {
-	lua_getglobal(L, "debug");
-	if (!lua_istable(L, -1)) {
-		lua_pop(L, 1);
-		return 1;
-	}
-	lua_getfield(L, -1, "traceback");
-	if (!lua_isfunction(L, -1)) {
-		lua_pop(L, 2);
-		return 1;
-	}
-	lua_pushvalue(L, 1);
-	lua_pushinteger(L, 2);
-	lua_call(L, 2, 1);
-	return 1;
+	return result;
 }
 
 int script_exception_wrapper(lua_State *L, lua_CFunction f)
@@ -110,7 +85,7 @@ void script_error(lua_State *L, int pcall_result, const char *mod, const char *f
 		err_descr = "<no description>";
 
 	char buf[256];
-	snprintf(buf, sizeof(buf), "%s error from mod '%s' in callback %s(): ",
+	porting::mt_snprintf(buf, sizeof(buf), "%s error from mod '%s' in callback %s(): ",
 		err_type, mod, fxn);
 
 	std::string err_msg(buf);
