@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef S_ENV_H_
-#define S_ENV_H_
+#pragma once
 
 #include "cpp_api/s_base.h"
 #include "irr_v3d.h"
@@ -44,5 +43,3 @@ public:
 
 	void initializeEnvironment(ServerEnvironment *env);
 };
-
-#endif /* S_ENV_H_ */
