@@ -27,6 +27,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "util/pointedthing.h"
 
 
+// Should be ordered exactly like enum NodeDrawType in nodedef.h
 struct EnumString ScriptApiNode::es_DrawType[] =
 	{
 		{NDT_NORMAL, "normal"},
@@ -34,18 +35,19 @@ struct EnumString ScriptApiNode::es_DrawType[] =
 		{NDT_LIQUID, "liquid"},
 		{NDT_FLOWINGLIQUID, "flowingliquid"},
 		{NDT_GLASSLIKE, "glasslike"},
-		{NDT_GLASSLIKE_FRAMED, "glasslike_framed"},
-		{NDT_GLASSLIKE_FRAMED_OPTIONAL, "glasslike_framed_optional"},
 		{NDT_ALLFACES, "allfaces"},
 		{NDT_ALLFACES_OPTIONAL, "allfaces_optional"},
 		{NDT_TORCHLIKE, "torchlike"},
 		{NDT_SIGNLIKE, "signlike"},
 		{NDT_PLANTLIKE, "plantlike"},
-		{NDT_FIRELIKE, "firelike"},
 		{NDT_FENCELIKE, "fencelike"},
 		{NDT_RAILLIKE, "raillike"},
 		{NDT_NODEBOX, "nodebox"},
+		{NDT_GLASSLIKE_FRAMED, "glasslike_framed"},
+		{NDT_FIRELIKE, "firelike"},
+		{NDT_GLASSLIKE_FRAMED_OPTIONAL, "glasslike_framed_optional"},
 		{NDT_MESH, "mesh"},
+		{NDT_PLANTLIKE_ROOTED, "plantlike_rooted"},
 		{0, NULL},
 	};
 
@@ -91,12 +93,6 @@ struct EnumString ScriptApiNode::es_NodeBoxType[] =
 		{0, NULL},
 	};
 
-ScriptApiNode::ScriptApiNode() {
-}
-
-ScriptApiNode::~ScriptApiNode() {
-}
-
 bool ScriptApiNode::node_on_punch(v3s16 p, MapNode node,
 		ServerActiveObject *puncher, PointedThing pointed)
 {
@@ -104,7 +100,7 @@ bool ScriptApiNode::node_on_punch(v3s16 p, MapNode node,
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "on_punch", &p))
@@ -127,7 +123,7 @@ bool ScriptApiNode::node_on_dig(v3s16 p, MapNode node,
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "on_dig", &p))
@@ -148,7 +144,7 @@ void ScriptApiNode::node_on_construct(v3s16 p, MapNode node)
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "on_construct", &p))
@@ -166,7 +162,7 @@ void ScriptApiNode::node_on_destruct(v3s16 p, MapNode node)
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "on_destruct", &p))
@@ -184,7 +180,7 @@ bool ScriptApiNode::node_on_flood(v3s16 p, MapNode node, MapNode newnode)
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "on_flood", &p))
@@ -196,7 +192,7 @@ bool ScriptApiNode::node_on_flood(v3s16 p, MapNode node, MapNode newnode)
 	pushnode(L, newnode, ndef);
 	PCALL_RES(lua_pcall(L, 3, 1, error_handler));
 	lua_remove(L, error_handler);
-	return (bool) lua_isboolean(L, -1) && (bool) lua_toboolean(L, -1) == true;
+	return readParam<bool>(L, -1, false);
 }
 
 void ScriptApiNode::node_after_destruct(v3s16 p, MapNode node)
@@ -205,7 +201,7 @@ void ScriptApiNode::node_after_destruct(v3s16 p, MapNode node)
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "after_destruct", &p))
@@ -224,7 +220,7 @@ bool ScriptApiNode::node_on_timer(v3s16 p, MapNode node, f32 dtime)
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// Push callback function on stack
 	if (!getItemCallback(ndef->get(node).name.c_str(), "on_timer", &p))
@@ -235,7 +231,7 @@ bool ScriptApiNode::node_on_timer(v3s16 p, MapNode node, f32 dtime)
 	lua_pushnumber(L,dtime);
 	PCALL_RES(lua_pcall(L, 2, 1, error_handler));
 	lua_remove(L, error_handler);
-	return (bool) lua_isboolean(L, -1) && (bool) lua_toboolean(L, -1) == true;
+	return readParam<bool>(L, -1, false);
 }
 
 void ScriptApiNode::node_on_receive_fields(v3s16 p,
@@ -247,10 +243,10 @@ void ScriptApiNode::node_on_receive_fields(v3s16 p,
 
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
-	INodeDefManager *ndef = getServer()->ndef();
+	const NodeDefManager *ndef = getServer()->ndef();
 
 	// If node doesn't exist, we don't know what callback to call
-	MapNode node = getEnv()->getMap().getNodeNoEx(p);
+	MapNode node = getEnv()->getMap().getNode(p);
 	if (node.getContent() == CONTENT_IGNORE)
 		return;
 
@@ -274,27 +270,3 @@ void ScriptApiNode::node_on_receive_fields(v3s16 p,
 	PCALL_RES(lua_pcall(L, 4, 0, error_handler));
 	lua_pop(L, 1);  // Pop error handler
 }
-
-void ScriptApiNode::node_falling_update(v3s16 p)
-{
-	SCRIPTAPI_PRECHECKHEADER
-
-	int error_handler = PUSH_ERROR_HANDLER(L);
-
-	lua_getglobal(L, "nodeupdate");
-	push_v3s16(L, p);
-	PCALL_RES(lua_pcall(L, 1, 0, error_handler));
-	lua_pop(L, 1);  // Pop error handler
-}
-
-void ScriptApiNode::node_falling_update_single(v3s16 p)
-{
-	SCRIPTAPI_PRECHECKHEADER
-
-	int error_handler = PUSH_ERROR_HANDLER(L);
-
-	lua_getglobal(L, "nodeupdate_single");
-	push_v3s16(L, p);
-	PCALL_RES(lua_pcall(L, 1, 0, error_handler));
-	lua_pop(L, 1);  // Pop error handler
-}
