@@ -113,7 +113,6 @@ void ScriptApiSecurity::initializeSecurity()
 		"setupvalue",
 		"setmetatable",
 		"upvalueid",
-		"upvaluejoin",
 		"sethook",
 		"debug",
 		"setlocal",
@@ -244,6 +243,7 @@ void ScriptApiSecurity::initializeSecurityClient()
 		"rawset",
 		"select",
 		"setfenv",
+		// getmetatable can be used to escape the sandbox
 		"setmetatable",
 		"tonumber",
 		"tostring",
@@ -265,6 +265,7 @@ void ScriptApiSecurity::initializeSecurityClient()
 	};
 	static const char *debug_whitelist[] = {
 		"getinfo",
+		"traceback"
 	};
 
 #if USE_LUAJIT
