@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef L_INVENTORY_H_
-#define L_INVENTORY_H_
+#pragma once
 
 #include "lua_api/l_base.h"
 
@@ -107,7 +106,7 @@ private:
 public:
 	InvRef(const InventoryLocation &loc);
 
-	~InvRef();
+	~InvRef() = default;
 
 	// Creates an InvRef and leaves it on top of stack
 	// Not callable from Lua; all references are created on the C side.
@@ -121,10 +120,10 @@ class ModApiInventory : public ModApiBase {
 private:
 	static int l_create_detached_inventory_raw(lua_State *L);
 
+	static int l_remove_detached_inventory_raw(lua_State *L);
+
 	static int l_get_inventory(lua_State *L);
 
 public:
 	static void Initialize(lua_State *L, int top);
 };
-
-#endif /* L_INVENTORY_H_ */
