@@ -21,14 +21,14 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "l_sound.h"
 #include "l_internal.h"
 #include "common/c_content.h"
-#include "guiEngine.h"
+#include "gui/guiEngine.h"
 
 
 int ModApiSound::l_sound_play(lua_State *L)
 {
 	SimpleSoundSpec spec;
 	read_soundspec(L, 1, spec);
-	bool looped = lua_toboolean(L, 2);
+	bool looped = readParam<bool>(L, 2);
 
 	s32 handle = getGuiEngine(L)->playSound(spec, looped);
 
