@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef L_UTIL_H_
-#define L_UTIL_H_
+#pragma once
 
 #include "lua_api/l_base.h"
 
@@ -66,6 +65,9 @@ private:
 	// is_yes(arg)
 	static int l_is_yes(lua_State *L);
 
+	// is_nan(arg)
+	static int l_is_nan(lua_State *L);
+
 	// get_builtin_path()
 	static int l_get_builtin_path(lua_State *L);
 
@@ -106,5 +108,3 @@ public:
 
 	static void InitializeAsync(AsyncEngine &engine);
 };
-
-#endif /* L_UTIL_H_ */
