@@ -18,13 +18,12 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef CLIENT_SCRIPTING_H_
-#define CLIENT_SCRIPTING_H_
+#pragma once
 
 #include "cpp_api/s_base.h"
 #include "cpp_api/s_client.h"
+#include "cpp_api/s_modchannels.h"
 #include "cpp_api/s_security.h"
-#include "util/basic_macros.h"
 
 class Client;
 class LocalPlayer;
@@ -32,7 +31,8 @@ class Camera;
 class ClientScripting:
 	virtual public ScriptApiBase,
 	public ScriptApiSecurity,
-	public ScriptApiClient
+	public ScriptApiClient,
+	public ScriptApiModChannels
 {
 public:
 	ClientScripting(Client *client);
@@ -41,6 +41,4 @@ public:
 
 private:
 	virtual void InitializeModApi(lua_State *L, int top);
-	DISABLE_CLASS_COPY(ClientScripting);
 };
-#endif
