@@ -3,9 +3,7 @@
 
 needs_compile || exit 0
 
-if hash clang-tidy-5.0 2>/dev/null; then
-	CLANG_TIDY=clang-tidy-5.0
-else
+if [ -z "${CLANG_TIDY}" ]; then
 	CLANG_TIDY=clang-tidy
 fi
 
